/////////////////////////////////////////////////////////////////////////////
// Name:        ButtonPropDlg.cpp
// Purpose:     DVD menu button properties dialog
// Author:      Alex Thuering
// Created:		20.11.2003
// RCS-ID:      $Id: ButtonPropDlg.cpp,v 1.9 2004/03/01 19:05:09 ntalex Exp $
// Copyright:  (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#include "ButtonPropDlg.h"

ButtonPropDlg::ButtonPropDlg(wxWindow* parent,
 MenuButton* object, Menu *menu, DVD* dvd):
 PropDlg(parent), m_object(object), m_menu(menu), m_dvd(dvd)
{
  Create();
}

void ButtonPropDlg::CreatePropPanel(wxSizer* sizer)
{
  wxBoxSizer* sizer2 = new wxBoxSizer(wxVERTICAL);
  wxFlexGridSizer* grid = new wxFlexGridSizer(3, 2, 4, 16);
  wxArrayString actions;
  for (int tsi=0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++)
  {
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	for (int pgci=0; pgci<(int)ts->GetTitles().GetPgcs().Count(); pgci++)
	  actions.Add(wxString::Format(_T("jump title %d;"),pgci+1));
  }
  AddComboProp(grid, _("Action:"), m_object->GetAction(), actions);
  AddTextProp(grid, _("Text:"), m_object->GetParam(_T("text")));
  AddFontProp(grid, _("Font:"), m_object->GetParamFont(_T("text_font")));
  grid->AddGrowableCol(1);
  sizer2->Add(grid, 0, wxEXPAND|wxALL, 10);

  grid = new wxFlexGridSizer(3, 4, 4, 16);
  AddDirectionProp( grid, bdLEFT);
  AddDirectionProp( grid, bdRIGHT);
  AddDirectionProp( grid, bdUP);
  AddDirectionProp( grid, bdDOWN);
  sizer2->Add(grid, 0, wxEXPAND|wxALL, 10);
  
  grid = new wxFlexGridSizer(3, 4, 4, 16);
  AddText(grid, _T(""));
  AddText(grid, _("Normal"));
  AddText(grid, _("Highlighted"));
  AddText(grid, _("Selected"));
  AddColourProp(grid, _("Text colour:"), m_object->GetParamColour(_T("text_colour")));
  AddColourProp(grid, _T(""), m_object->GetParamColour(_T("text_colour"), mbsHIGHLIGHTED));
  AddColourProp(grid, _T(""), m_object->GetParamColour(_T("text_colour"), mbsSELECTED));
  AddColourProp(grid, _("BG colour:"), m_object->GetBackgroundColour());
  AddColourProp(grid, _T(""), m_object->GetBackgroundColour(mbsHIGHLIGHTED));
  AddColourProp(grid, _T(""), m_object->GetBackgroundColour(mbsSELECTED));
  sizer2->Add(grid, 0, wxEXPAND|wxALL, 10);
  sizer2->Add(10, 10, 1, wxEXPAND);
  sizer->Add(sizer2, 1, wxEXPAND|wxALL, 10);
}

void ButtonPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir)
{
	wxString label;
	wxArrayString buttons;
  	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
	
	switch ( dir)
	{
		case bdLEFT:
			label = _("Left");
			break;
		case bdRIGHT:
			label = _("Right");
			break;
		case bdUP:
			label = _("Up");
			break;
		case bdDOWN:
			label = _("Down");
			break;
		case bdNONE:
			return;
	}
	
	// add all buttons except current
	buttons.Add(_("None"));
	int btInd = 1;
	for ( int i=0; i<m_menu->GetObjectsCount(); i++)
	{
		MenuObject* obj = m_menu->GetObject(i);
		if ( obj->GetType() == moBUTTON)
		{
			if ( obj != m_object)
			{
				wxString text = ((MenuButton*) obj)->GetParam(_T("text"));
				if (!text.length())
				  text = wxString::Format(_T("button%d"), btInd);
				buttons.Add(text);
			}
			else
			  m_btInd = btInd-1;
			btInd++;
		}
	}
	int defValue = m_object->GetDirection(dir-bdLEFT)+1;
	if (defValue>m_btInd)
	  defValue--;
  	AddComboProp(grid, label, buttons[defValue], buttons, wxCB_READONLY);
	sizer->Add(grid, 0, 0, 0);
}

bool ButtonPropDlg::SetValues()
{
  m_object->SetAction(GetString(0));
  
  m_object->SetParam(_T("text"), GetString(1));
  m_object->SetParamFont(_T("text_font"), GetFont(2).GetChosenFont());
  
  int btInd = GetInt(3)-1;
  if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(0, btInd);
  btInd = GetInt(4)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(1, btInd);
  btInd = GetInt(5)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(2, btInd);
  btInd = GetInt(6)-1; if (btInd >= m_btInd) btInd++;
  m_object->SetDirection(3, btInd);
  
  m_object->SetParamColour(_T("text_colour"), GetColour(7));
  m_object->SetParamColour(_T("text_colour"), GetColour(8), mbsHIGHLIGHTED);
  m_object->SetParamColour(_T("text_colour"), GetColour(9), mbsSELECTED);
  m_object->SetBackgroundColour(GetColour(10));
  m_object->SetBackgroundColour(GetColour(11), mbsHIGHLIGHTED);
  m_object->SetBackgroundColour(GetColour(12), mbsSELECTED);
  return true;
}

