/////////////////////////////////////////////////////////////////////////////
// Name:        Config.h
// Purpose:     Configuration
// Author:      Alex Thuering
// Created:		27.03.2003
// RCS-ID:      $Id: Config.h,v 1.12 2004/08/26 11:29:51 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef DS_CONFIG_H
#define DS_CONFIG_H

#include <wx/wx.h>
#include <wx/config.h>

const wxString DEF_VERSION      = _T("");
const wxString DEF_LANGUAGE     = _T("");
const int DEF_MENU_FORMAT       = 0;
const wxString DEF_GEN_TEMP_DIR = wxGetHomeDir() + wxFILE_SEP_PATH + _T("dvd");
const wxString DEF_DEMPLEX_CMD  = _T("mpgtx -f -d \"$FILE_IN\" -b \"$FILES_OUT\"");
#ifdef __WXMSW__
  const wxString DEF_JPEG2MPEG_CMD = _T("jpeg2yuv -n 1 -I t -L 1 -f $FRAME_RATE -j '$FILE_IN' | mpeg2enc -f 8 -o '$FILE_OUT' $VIDEO_NORM");
  const wxString DEF_FRAMERATE_PAL = _T("25");
  const wxString DEF_FRAMERATE_NTSC = _T("29.97");
  const wxString DEF_MPLEX_CMD    = _T("mplex -f 8 -o \"$FILE_OUT\" \"$FILE_VIDEO\" \"$FILE_AUDIO\"");
  const wxString DEF_SPUMUX_CMD   = _T("spumux '$FILE_CONF'");
  const wxString DEF_DVDAUTHOR_CMD = _T("dvdauthor -o '$DIR' -x '$FILE_CONF'");
  const bool DEF_PREVIEW_DO        = false;
  const wxString DEF_PREVIEW_CMD   = _T("wmplayer '$DIR\\VIDEO_TS\\VIDEO_TS.VOB'");
  const bool DEF_ISO_DO            = true;
  const wxString DEF_ISO_CMD       = _T("mkisofs -o '$FILE' -dvd-video '$DIR'");
  const bool DEF_BURN_DO           = false;
  const wxString DEF_BURN_CMD      = _T("mkisofs -dvd-video '$DIR' | cdrecord-prodvd.exe dev=$DEV -v driveropts=burnfree fs=64m -dao tsize=$SIZEs -");
  const wxString DEF_BURN_DEVICE   = _T("1,0,0");
  const bool DEF_FORMAT_DO         = false;
  const wxString DEF_FORMAT_CMD    = _T("");
  const wxString DEF_CDR_SECURITY  = _T("8:dvd,clone:sparc-sun-solaris2,i386-pc-solaris2,i586-pc-linux,powerpc-apple,hppa,powerpc-ibm-aix,i386-unknown-freebsd,i386-unknown-openbsd,i386-unknown-netbsd,powerpc-apple-netbsd,i386-pc-bsdi,mips-sgi-irix,i386-pc-sco:1.11::1093000000:::private/research/educational_non-commercial_use:9vl2T2kP6w6O4h.bXuet8hP1Z3H5erm3qWmxhbcr.fHvuN8ZJbhQUWBzjAc");
#else
  const wxString DEF_JPEG2MPEG_CMD = _T("jpegtopnm \"$FILE_IN\" | ppmtoy4m -n 1 -I t -L $FRAME_RATE | mpeg2enc -f 8 -o \"$FILE_OUT\" $VIDEO_NORM");
  const wxString DEF_FRAMERATE_PAL = _T("-F25:1 -A 59:54");
  const wxString DEF_FRAMERATE_NTSC = _T("-F30000:1001 -A 10:11");
  const wxString DEF_MPLEX_CMD    = _T("mplex -f 8 -o \"$FILE_OUT\" \"$FILE_VIDEO\" \"$FILE_AUDIO\"");
  const wxString DEF_SPUMUX_CMD   = _T("spumux \"$FILE_CONF\"");
  const wxString DEF_DVDAUTHOR_CMD = _T("dvdauthor -o \"$DIR\" -x \"$FILE_CONF\"");
  const bool DEF_PREVIEW_DO        = true;
  const wxString DEF_PREVIEW_CMD   = _T("xine \"dvd:/$DIR\"");
  const bool DEF_ISO_DO            = false;
  const wxString DEF_ISO_CMD       = _T("mkisofs -o \"$FILE\" -dvd-video \"$DIR\"");
  const bool DEF_BURN_DO           = true;
  const wxString DEF_BURN_CMD      = _T("mkisofs -dvd-video \"$DIR\" | growisofs -Z $DEV=/dev/fd/0");
  const wxString DEF_BURN_DEVICE   = _T("/dev/dvd");
  const wxString DEF_CDR_SECURITY  = _T("");
  const bool DEF_FORMAT_DO         = false;
  const wxString DEF_FORMAT_CMD    = _T("dvd+rw-format -force $DEV");
#endif
const wxString DEF_ISO_SAVETO   = wxGetHomeDir() + wxFILE_SEP_PATH + _T("dvd.iso");

#define CONFIG_PROP(name, cfgName, defName)\
  CONFIG_PROP_T(name, cfgName, defName, wxString)

#define CONFIG_PROP_BOOL(name, cfgName, defName)\
  CONFIG_PROP_T(name, cfgName, defName, bool)

#define CONFIG_PROP_INT(name, cfgName, defName)\
  CONFIG_PROP_T(name, cfgName, defName, int)

#define CONFIG_PROP_T(name, cfgName, defName, cfgType)\
  cfgType Get##name(bool def = false)\
  { return def ? defName : cfg->Read(cfgName, defName); }\
  void Set##name(cfgType value)\
  {\
    wxLogNull log;\
    if (value == defName) cfg->DeleteEntry(cfgName);\
    else cfg->Write(cfgName, value);\
  }

class Config
{
  public:
    void Init() { cfg = wxConfig::Get(); }
    bool Flush() { return cfg->Flush(); }
    bool DeleteAll() { return cfg->DeleteAll(); }
    bool DeleteGroup(wxString group) { return cfg->DeleteGroup(group); }
    
    wxRect GetMainWinLocation();
    void SetMainWinLocation(wxRect rect);
	
	CONFIG_PROP(Version,  _T("Version"), DEF_VERSION)
	CONFIG_PROP(Language, _T("Interface/Language"), DEF_LANGUAGE)
	CONFIG_PROP_INT(DefMenuFormat, _T("Interface/DefMenuFormat"), DEF_MENU_FORMAT)
	CONFIG_PROP(GenerateTemp, _T("Generate/TempDir"), DEF_GEN_TEMP_DIR)
	CONFIG_PROP(Jpeg2MpegCmd, _T("Generate/Jpeg2MpegCmd"), DEF_JPEG2MPEG_CMD)
	CONFIG_PROP(FrameRatePAL, _T("Generate/FrameRatePAL"), DEF_FRAMERATE_PAL)
	CONFIG_PROP(FrameRateNTSC,_T("Generate/FrameRateNTSC"), DEF_FRAMERATE_NTSC)
	CONFIG_PROP(MplexCmd,   _T("Generate/MplexCmd"), DEF_MPLEX_CMD)
	CONFIG_PROP(DemplexCmd, _T("Generate/DemplexCmd"), DEF_DEMPLEX_CMD)
	CONFIG_PROP(SpumuxCmd,  _T("Generate/SpumuxCmd"), DEF_SPUMUX_CMD)
	CONFIG_PROP(DvdauthorCmd, _T("Generate/DvdauthorCmd"), DEF_DVDAUTHOR_CMD)
	
	CONFIG_PROP_BOOL(PreviewDo, _T("Preview/Do"), DEF_PREVIEW_DO)
	CONFIG_PROP(PreviewCmd, _T("Preview/Cmd"), DEF_PREVIEW_CMD)
	
	CONFIG_PROP_BOOL(IsoDo, _T("Iso/Do"), DEF_ISO_DO)
	CONFIG_PROP(IsoCmd, _T("Iso/Cmd"), DEF_ISO_CMD)
	CONFIG_PROP(IsoSaveTo, _T("Iso/SaveTo"), DEF_ISO_SAVETO)
	
	CONFIG_PROP_BOOL(BurnDo, _T("Burn/Do"), DEF_BURN_DO)
	CONFIG_PROP(BurnCmd, _T("Burn/Cmd"), DEF_BURN_CMD)
	CONFIG_PROP(BurnDevice, _T("Burn/Device"), DEF_BURN_DEVICE)
	CONFIG_PROP(CdrSecurity, _T("Burn/CdrSecurity"), DEF_CDR_SECURITY)
	
	CONFIG_PROP_BOOL(FormatDo, _T("Burn/FormatDo"), DEF_FORMAT_DO)
	CONFIG_PROP(FormatCmd, _T("Burn/FormatCmd"), DEF_FORMAT_CMD)
    
  protected:
    wxConfigBase* cfg;
};

extern Config s_config;

#endif //DS_CONFIG_H
