/////////////////////////////////////////////////////////////////////////////
// Name:        DVD.cpp
// Purpose:     The class to store a DVD Structure (Titles/Menus)
// Author:      Alex Thuering
// Created:		29.01.2003
// RCS-ID:      $Id: DVD.cpp,v 1.6 2004/07/22 23:54:12 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "DVD.h"
#include <wxXML/xmlhelpr.h>

////////////////////////////////// Vob //////////////////////////////////////

wxXmlNode* Vob::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("vob"));
  node->AddProperty(_T("file"),
    type == DVDAUTHOR_XML && GetTmpFilename().length() ?
	GetTmpFilename() : GetFilename());
  if (GetPause() != 0)
  {
	wxString pause = _T("inf");
	if (GetPause()>0)
	  pause = wxString::Format(_T("%d"), GetPause());
	node->AddProperty(_T("pause"), pause);
  }
  if (GetChapters().length())
	node->AddProperty(_T("chapters"), GetChapters());
  return node;
}

bool Vob::PutXML(wxXmlNode* node)
{
  wxString val;
  long lval;
  
  if (node->GetPropVal(_T("file"), &val))
	m_filename = val;
  else if (!m_menu)
  {
	wxLogError(_("Can't load vob: filename must not be empty"));
	return false;
  }
  if (node->GetPropVal(_T("pause"), &val) && val.ToLong(&lval))
	m_pause = int(lval);
  if (node->GetPropVal(_T("chapters"), &val))
	m_chapters = val;
  return true;
}

////////////////////////////////// Pgc //////////////////////////////////////

Menu* Pgc::GetMenu()
{
  for (int i=0; i<(int)GetVobs().GetCount(); i++)
	if (GetVobs()[i]->GetMenu())
	  return GetVobs()[i]->GetMenu();
  return NULL;
}

wxXmlNode* Pgc::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("pgc"));
  
  Menu* menu = GetMenu();
  if (menu)
  {
	if (type == DVDSTYLER_XML) // DVDSTYLER_XML
	{
	  wxXmlNode* spuNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("spumux"));
	  spuNode->AddChild(menu->GetXML(type));
	  node->AddChild(spuNode);
	}
	else if (type == DVDAUTHOR_XML) // DVDAUTHOR_XML
	  menu->GetXML(type, node);
  }
  
  // DVDAUTHOR_XML & DVDSTYLER_XML
  for (int i=0; i<(int)GetVobs().GetCount(); i++)
	node->AddChild(GetVobs()[i]->GetXML(type));
  if (GetPreCommands().length())
	XmlWriteValue(node, _T("pre"), GetPreCommands());
  if (GetPostCommands().length())
	XmlWriteValue(node, _T("post"), GetPostCommands());
  
  return node;
}

bool Pgc::PutXML(wxXmlNode* node)
{
  WX_CLEAR_ARRAY(GetVobs());
  wxXmlNode* child = node->GetChildren();
  Menu* menu = NULL;
	
  while (child)
  {
	  // get spunode
	  if (child->GetName() == _T("spumux"))
	  {
		  // get menu
		  menu = new Menu();
		  menu->PutXML(child);
	  }
	  // get dvdauth nodes
	  else if (child->GetName() == _T("vob"))
	  {
		  Vob* vob;
		  if (menu)
			  vob = new Vob(menu);
		  else
			  vob = new Vob(_T(""));
		  if (vob->PutXML(child))
			  GetVobs().Add(vob);
		  else
		  {
			  delete vob;
			  wxLogError(_("Can't load vob"));
			  return false;
		  }
	  }
	  // get pre commands
	  else if (child->GetName() == _T("pre"))
		SetPreCommands(child->GetChildren()->GetContent());
	  // get post commands
	  else if (child->GetName() == _T("post"))
		SetPostCommands(child->GetChildren()->GetContent());
	  // get buttons to retrieve actions
	  else if (child->GetName() == _T("button"))
	  {
	  }		
	  child = child->GetNext();
  }
  return true;
}
 
////////////////////////////////// Menus //////////////////////////////////////

wxXmlNode* Menus::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("menus"));
  for (int i=0; i<(int)m_pgcs.GetCount(); i++)
	node->AddChild(m_pgcs[i]->GetXML(type));
  return node;
}

bool Menus::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("menus"))
	return false;
  WX_CLEAR_ARRAY(GetPgcs());
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	Pgc* pgc = new Pgc;
	if (pgc->PutXML(child))
	  GetPgcs().Add(pgc);
	else
	{
	  delete pgc;
	  wxLogError(_("Can't load pgc"));
	  return false;
	}
	child = child->GetNext();
  }
  return true;
}

////////////////////////////////// Titles ////////////////////////////////////

wxXmlNode* Titles::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("titles"));
  for (int i=0; i<(int)m_pgcs.GetCount(); i++)
	node->AddChild(m_pgcs[i]->GetXML(type));
  return node;
}

bool Titles::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("titles"))
	return false;
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	if (child->GetName() == _T("pgc"))
	{
	  Pgc* pgc = new Pgc;
	  if (pgc->PutXML(child))
		GetPgcs().Add(pgc);
	  else
	  {
		delete pgc;
		wxLogError(_("Can't load pgc"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

////////////////////////////////// Titleset //////////////////////////////////

wxXmlNode* Titleset::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("titleset"));
  node->AddChild(m_menus.GetXML(type));
  node->AddChild(m_titles.GetXML(type));
  return node;
}

bool Titleset::PutXML(wxXmlNode* node)
{
  if (node == NULL || node->GetName() !=  _T("titleset"))
	return false;
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
	if (child->GetName() == _T("menus"))
	{
	  if (!GetMenus().PutXML(child))
	  {
		wxLogError(_("Can't load menus"));
		return false;
	  }
	}
	else if (child->GetName() == _T("titles"))
	{
	  if (!GetTitles().PutXML(child))
	  {
		wxLogError(_("Can't load titles"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

////////////////////////////////// DVD //////////////////////////////////////

bool DVD::Open(wxString fname)
{
  WX_CLEAR_ARRAY(GetTitlesets());
  wxXmlDocument xml;
  if (!xml.Load(fname))
  {
	wxLogError(_("Can't load xml-file"));
	return false;
  }
  
  wxXmlNode* root = xml.GetRoot();
  if (root == NULL || root->GetName() !=  _T("dvdstyler"))
  {
	wxLogError(_("This is not a DVDStyler project file"));
	return false;
  }
  
  wxXmlNode* child = root->GetChildren();
  while (child)
  {
	if (child->GetName() ==  _T("vmgm"))
	{
	  if (!GetVmgm().PutXML(child->GetChildren()))
	  {
		wxLogError(_("Can't load vmgm menus"));
		return false;
	  }
	}
	else if (child->GetName() ==  _T("titleset"))
	{
	  Titleset* titleset = new Titleset;
	  if (titleset->PutXML(child))
		GetTitlesets().Add(titleset);
	  else
	  {
		delete titleset;
		wxLogError(_("Can't load titleset"));
		return false;
	  }
	}
	child = child->GetNext();
  }
  return true;
}

bool DVD::Save(wxString fname)
{
  	wxXmlDocument xml;
	wxXmlNode* child;
	wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("dvdstyler"));
	wxXmlNode* vmgmNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("vmgm"));
	child = GetVmgm().GetXML(DVDSTYLER_XML);
	if (child)
		vmgmNode->AddChild(child);
	root->AddChild(vmgmNode);
	for (int i=0; i<(int)m_titlesets.GetCount(); i++)
	{
		child = m_titlesets[i]->GetXML(DVDSTYLER_XML);
		root->AddChild(child);
	}
	xml.SetRoot(root);
	return xml.Save(fname);
}

bool DVD::SaveDVDAuthor(wxString fname)
{
	wxXmlDocument xml;
	wxXmlNode* child;
	wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("dvdauthor"));
	wxXmlNode* vmgmNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("vmgm"));
	child = GetVmgm().GetXML(DVDAUTHOR_XML);
	if (child)
		vmgmNode->AddChild(child);
	root->AddChild(vmgmNode);
	for (int i=0; i<(int)m_titlesets.GetCount(); i++)
	{
		child = m_titlesets[i]->GetXML(DVDAUTHOR_XML);
		if (child)
			root->AddChild(child);
	}
	xml.SetRoot(root);
	return xml.Save(fname);
}

