/////////////////////////////////////////////////////////////////////////////
// Name:        DVD.h
// Purpose:     The class to store a DVD Structure (Titles/Menus)
// Author:      Alex Thuering
// Created:		29.01.2003
// RCS-ID:      $Id: DVD.h,v 1.4 2004/07/22 23:54:11 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef DVDAUTHOR_H
#define DVDAUTHOR_H

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/dynarray.h>
#include <wxXML/xmlhelpr.h>
#include "Menu.h"
	
class Vob
{
  public:
	Vob(wxString filename) { m_filename = filename;m_pause = 0;m_menu = NULL; }
	Vob(Menu* menu) { m_pause = -1; m_menu = menu; }
	~Vob() { if (m_menu) delete m_menu; }
	
	void SetFilename(wxString value) { m_filename = value; }
	wxString GetFilename() { return m_filename; }
	
	void SetTmpFilename(wxString value) { m_tmpFilename = value; }
	wxString GetTmpFilename() { return m_tmpFilename; }
	
	void SetChapters(wxString value) { m_chapters = value; }
	wxString GetChapters() { return m_chapters; }
	
	void SetPause(int value) { m_pause = value; }
	int GetPause() { return m_pause; }
	
	Menu* GetMenu() { return m_menu; }
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
  protected:
	wxString m_filename;
	wxString m_tmpFilename;
	wxString m_chapters;
	int m_pause;
	Menu* m_menu;
};

WX_DEFINE_ARRAY(Vob*, VobArray);

class Pgc
{
  public:
	Pgc() { }
	~Pgc() { WX_CLEAR_ARRAY(m_vobs) }
	
	void SetPreCommands(wxString value) { m_pre = value; }
	wxString GetPreCommands() { return m_pre; }
	void SetPostCommands(wxString value) { m_post = value; }
	wxString GetPostCommands() { return m_post; }
	
	VobArray& GetVobs() { return m_vobs; }
	
	Menu* GetMenu();
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
  protected:
	VobArray m_vobs;
	wxString m_pre, m_post;
};

WX_DEFINE_ARRAY(Pgc*, PgcArray);

class Menus
{
  public:
	Menus() {}
	~Menus() { WX_CLEAR_ARRAY(m_pgcs); }
	
	PgcArray& GetPgcs() { return m_pgcs; }
	int	GetPgcArrayCount()	{ return m_pgcs.Count();	}
	
	wxXmlNode*	GetXML(DVDFileType type);
	bool		PutXML(wxXmlNode* node);
	
  protected:
	PgcArray m_pgcs;
};

class Titles
{
  public:
	Titles() {}
	~Titles() { WX_CLEAR_ARRAY(m_pgcs); }
	
	PgcArray& GetPgcs() { return m_pgcs; }
	
	wxXmlNode* GetXML(DVDFileType type);
	bool	   PutXML(wxXmlNode* node);
	
  protected:
	PgcArray m_pgcs;
};

class Titleset
{
  public:
	Titleset() {}
	
	Menus& GetMenus() { return m_menus; }
	Titles& GetTitles() { return m_titles; }
	
	wxXmlNode* GetXML(DVDFileType type);
	bool	   PutXML(wxXmlNode* node);
	
  protected:
	Menus m_menus;
	Titles m_titles;
};

WX_DEFINE_ARRAY(Titleset*, TitlesetArray);

class DVD
{
  public:
	DVD() {}
	~DVD() { WX_CLEAR_ARRAY(m_titlesets) }

	TitlesetArray& GetTitlesets() { return m_titlesets; }
	Menus& GetVmgm() { return m_vmgm; }
	int	GetTitleSetsCount()	{ return m_titlesets.Count();	}
	Menus* GetMenus()	{ return &m_vmgm; }
	
	bool Open(wxString fname);
	bool Save(wxString fname);
	bool SaveDVDAuthor(wxString fname);
	
  protected:
	TitlesetArray m_titlesets;
	Menus m_vmgm;
};

#endif //DVDAUTHOR_H
