/////////////////////////////////////////////////////////////////////////////
// Name:        Languages.cpp
// Purpose:     Languages dialog
// Author:      Alex Thuering
// Created:		23.05.2003
// Remarks:		Based on LangChoice from audacity
// RCS-ID:      $Id: Languages.cpp,v 1.5 2004/07/04 08:22:40 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "Languages.h"
#include <wx/defs.h>
#include <wx/button.h>
#include <wx/choice.h>
#include <wx/dialog.h>
#include <wx/intl.h>
#include <wx/sizer.h>
#include <wx/stattext.h>

// languages supported by dvdstyler
// consts: langCount, langDefault
// arrays: langNames, langCodes
#include "../locale/langs.h"

wxArrayString GetLangNames()
{
  wxArrayString langsArr;
  for (int i=0; i<langCount; i++)
	langsArr.Add(langNames[i]);
  return langsArr;
}

wxString GetLangName(wxString code)
{
  for (int i=0; i<langCount; i++)
	if (code == langCodes[i])
	  return langNames[i];
  return langNames[langDefault];
}

wxString GetLangCode(int index)
{
  return langCodes[index];
}


class LanguageDlg: public wxDialog
{
  public:
	LanguageDlg(wxWindow* parent);
	wxString GetLanguage() { return m_lang; }
	
  private:
	void OnOk(wxCommandEvent & event);
	
	wxChoice *m_choice;
	wxString m_lang;
	
	DECLARE_EVENT_TABLE()
};

int GetSystemLanguageCode()
{
  int sysLang = wxLocale::GetSystemLanguage();
  const wxLanguageInfo *info = wxLocale::GetLanguageInfo(sysLang);
  
  if (!info || info->CanonicalName.length()<2)
   return langDefault;
   
  wxString fullCode = info->CanonicalName;
  wxString code = fullCode.Left(2);

  for (int i=0; i<langCount; i++)
	if (langCodes[i] == fullCode || langCodes[i] == code)
	  return i;
  
  return langDefault;
}

wxString ChooseLanguage(wxWindow *parent)
{
  wxSingleChoiceDialog dlg(parent, _T("Choose Language for DVDStyler to use:"),
    _T("DVDStyler First Run"), langCount, langNames, NULL, wxOK|wxCENTRE);
  dlg.SetSelection(GetSystemLanguageCode());
  dlg.ShowModal();
  return langCodes[dlg.GetSelection()];
  /*LanguageDlg dlg(parent);
  dlg.ShowModal();
  return dlg.GetLanguage();*/
}

BEGIN_EVENT_TABLE(LanguageDlg, wxDialog)
  EVT_BUTTON(wxID_OK, LanguageDlg::OnOk)
END_EVENT_TABLE()

LanguageDlg::LanguageDlg(wxWindow* parent):
   wxDialog(parent, -1, wxString(_T("DVDStyler First Run")))
{
  wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *hSizer = new wxBoxSizer(wxHORIZONTAL);
  
  hSizer->Add(
    new wxStaticText(this, -1, _T("Choose Language for DVDStyler to use:")),
	0, wxALIGN_CENTRE | wxALIGN_CENTER_VERTICAL | wxALL, 8);
  
  m_choice = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize,
	langCount, langNames);
  m_choice->SetSelection(GetSystemLanguageCode());
  
  hSizer->Add(m_choice, 0, wxALIGN_CENTRE | wxALIGN_CENTER_VERTICAL | wxALL, 8);
  mainSizer->Add(hSizer, 0, wxALL, 8);
  
  wxButton *okBt = new wxButton(this, wxID_OK, _("OK"));
  okBt->SetDefault();
  mainSizer->Add(okBt, 0, wxALIGN_CENTRE | wxALL, 8);
  
  SetAutoLayout(true);
  SetSizer(mainSizer);
  mainSizer->Fit(this);
  mainSizer->SetSizeHints(this);
  Centre();
}

void LanguageDlg::OnOk(wxCommandEvent & event)
{
  m_lang = langCodes[m_choice->GetSelection()];
  EndModal(true);
}

