/////////////////////////////////////////////////////////////////////////////
// Name:        MainWin.cpp
// Purpose:     Main window
// Author:      Alex Thuering
// Created:		10.10.2003
// RCS-ID:      $Id: MainWin.cpp,v 1.41 2004/08/26 11:33:25 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "MainWin.h"
#include "BurnDlg.h"
#include "ProgressDlg.h"
#include "About.h"
#include "SettingsDlg.h"
#include "MPEG.h"
#include <wxVillaLib/VerticalToolbar.h>
#include <wxVillaLib/utils.h>
#include <wx/imaglist.h>
#include "math.h"

#if defined(__WXMSW__) && defined(__WIN95__)
#include <wx/msw/registry.h>
#undef  wxBITMAP
#define wxBITMAP(icon) wxBitmap(icon##_xpm)
#else
#define wxDirDialogDefaultFolderStr _T("")
#endif

#include "rc/new.xpm"
#include "rc/open.xpm"
#include "rc/save.xpm"
#include "rc/run.xpm"
#include "rc/minus.xpm"
#include "rc/plus.xpm"
#include "rc/folder1.xpm"
#include "rc/folder2.xpm"
#include "rc/files.xpm"


#define BACKGROUNDS_DIR wxGetAppPath() + _T("/../backgrounds")
#define BUTTONS_DIR wxGetAppPath() + _T("/../buttons")

//////////////////////////// help functions //////////////////////////////////

wxBitmap SetDefaultMask(wxBitmap bitmap)
{
  wxImage tmp = bitmap.ConvertToImage();
  wxColour cmask(tmp.GetRed(0,0), tmp.GetGreen(0,0), tmp.GetBlue(0,0));
  bitmap.SetMask(new wxMask(bitmap, cmask));
  return bitmap;
}

//////////////////////////////// MainWin /////////////////////////////////////

BEGIN_EVENT_TABLE(MainWin, wxFrame)
  EVT_TREE_SEL_CHANGED(-1, MainWin::OnDirSelected)
  
  EVT_THUMBNAILS_POINTED(-1, MainWin::OnThumbInfo)
  EVT_THUMBNAILS_DCLICK(-1,  MainWin::OnSetBackground)
  
  EVT_TOOL(DIR_TREE_BT_ID, MainWin::OnDirTreeBt)
  EVT_TOOL(BACKGROUNS_BT_ID, MainWin::OnBackgroundsBt)
  EVT_TOOL(BUTTONS_BT_ID, MainWin::OnButtonsBt)
  
  EVT_TOOL(wxID_NEW, MainWin::OnNew)
  EVT_TOOL(wxID_OPEN, MainWin::OnOpen)
  EVT_TOOL(wxID_SAVE, MainWin::OnSave)
  EVT_TOOL(wxID_SAVEAS, MainWin::OnSaveAs)
  EVT_TOOL(MENU_BURN_ID, MainWin::OnBurn)
  EVT_TOOL(wxID_EXIT, MainWin::OnExit)
  EVT_TOOL(MENU_SETTINGS_ID, MainWin::OnSettings)
  EVT_TOOL(wxID_ABOUT, MainWin::OnAbout)
  
  EVT_IMAGEBOX_IMAGE_CHANGED(-1, MainWin::OnChanged)
  EVT_DVD_CHANGED(-1, MainWin::OnChanged)
  
  EVT_COMBOBOX(SCALE_BOX_ID, MainWin::OnChangeScale)
  EVT_TEXT_ENTER(SCALE_BOX_ID, MainWin::OnChangeScaleText)
  EVT_TOOL(SCALE_PLUS_ID, MainWin::OnScalePlus)
  EVT_TOOL(SCALE_MINUS_ID, MainWin::OnScaleMinus)
  
  EVT_SIZE(MainWin::OnResize)
  EVT_SPLITTER_SASH_POS_CHANGED(SPLIITTER_TITLE_ID, MainWin::OnSplitterTitleSash)
  EVT_CLOSE(MainWin::OnClose)
END_EVENT_TABLE()

MainWin::MainWin(): wxFrame(NULL, -1, _T(""),
  wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE)
{
    // begin wxGlade: MainWin::MainWin
    m_splitterTitle = new wxSplitterWindow(this, SPLIITTER_TITLE_ID, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxCLIP_CHILDREN);
    m_splitterDir = new wxSplitterWindow(m_splitterTitle, -1, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxCLIP_CHILDREN);
    m_splitterThumb = new wxSplitterWindow(m_splitterDir, -1, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxCLIP_CHILDREN);
    m_thumbPanel = new wxPanel(m_splitterThumb, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxCLIP_CHILDREN);
    m_menubar = new wxMenuBar();
    SetMenuBar(m_menubar);
    wxMenu* wxglade_tmp_menu_1 = new wxMenu();
    wxglade_tmp_menu_1->Append(wxID_NEW, _("&New"), wxT(""), wxITEM_NORMAL);
    wxglade_tmp_menu_1->Append(wxID_OPEN, _("&Open..."), wxT(""), wxITEM_NORMAL);
    wxglade_tmp_menu_1->Append(wxID_SAVE, _("&Save"), wxT(""), wxITEM_NORMAL);
    wxglade_tmp_menu_1->Append(wxID_SAVEAS, _("S&ave as..."), wxT(""), wxITEM_NORMAL);
    wxglade_tmp_menu_1->AppendSeparator();
    wxglade_tmp_menu_1->Append(MENU_BURN_ID, _("&Burn dvd..."), wxT(""), wxITEM_NORMAL);
    wxglade_tmp_menu_1->AppendSeparator();
    wxglade_tmp_menu_1->Append(wxID_EXIT, _("&Exit"), wxT(""), wxITEM_NORMAL);
    m_menubar->Append(wxglade_tmp_menu_1, _("&File"));
    wxMenu* wxglade_tmp_menu_2 = new wxMenu();
    wxglade_tmp_menu_2->Append(MENU_SETTINGS_ID, _("&Settings..."), wxT(""), wxITEM_NORMAL);
    m_menubar->Append(wxglade_tmp_menu_2, _("&Configuration"));
    wxMenu* wxglade_tmp_menu_3 = new wxMenu();
    wxglade_tmp_menu_3->Append(wxID_ABOUT, _("&About..."), wxT(""), wxITEM_NORMAL);
    m_menubar->Append(wxglade_tmp_menu_3, _("&Help"));
    m_statusbar = CreateStatusBar(1, 0);
    m_dirBar = new wxToolBar(this, -1, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTB_VERTICAL|wxTB_FLAT);
    m_dirTree = new wxGenericDirCtrl(m_splitterDir, DIR_TREE_ID, wxDirDialogDefaultFolderStr, wxDefaultPosition, wxDefaultSize, wxDIRCTRL_DIR_ONLY );
    m_thumbLabel = new wxStaticText(m_thumbPanel, -1, _("Directory"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE|wxST_NO_AUTORESIZE);
    m_thumbnails = new wxThumbnails(m_thumbPanel, -1);
    m_menuBox = new MenuBox(m_splitterThumb, -1);
    m_titleThumbnails = new TitleThumbnails(m_splitterTitle, -1);

    set_properties();
    do_layout();
    // end wxGlade
	
	// set home dir (but not update thumbnails)
	wxString homeDir = wxGetHomeDir();
#ifdef __WXMSW__
	if (wxGetenv(_T("USERPROFILE")) != NULL)
      homeDir = wxGetenv(_T("USERPROFILE"));
#endif
	wxGenericDirCtrl* dirTreeTmp = m_dirTree; m_dirTree = NULL;
	dirTreeTmp->SetPath(homeDir); m_dirTree = dirTreeTmp;
	
	// show Backgrouns
	m_dirBar->ToggleTool(BACKGROUNS_BT_ID, true);
	wxCommandEvent evt;
	OnBackgroundsBt(evt);
	NewDVD((VideoFormat)s_config.GetDefMenuFormat());
	
	// Restore frame size/position
	wxRect rect = s_config.GetMainWinLocation();
	if (rect.width > 0 && rect.height > 0)
	  SetSize(rect);
}

void MainWin::set_properties()
{
    // begin wxGlade: MainWin::set_properties
    SetTitle(_("DVDStyler"));
    SetSize(wxSize(1000, 740));
    int m_statusbar_widths[] = { -1 };
    m_statusbar->SetStatusWidths(1, m_statusbar_widths);
    const wxString m_statusbar_fields[] = {
        wxT("")
    };
    for(int i = 0; i < m_statusbar->GetFieldsCount(); ++i) {
        m_statusbar->SetStatusText(m_statusbar_fields[i], i);
    }
    m_dirBar->SetSize(wxSize(40, -1));
    m_thumbLabel->SetBackgroundColour(wxColour(90, 16, 198));
    m_thumbLabel->SetForegroundColour(wxColour(255, 255, 255));
    m_titleThumbnails->SetSize(wxSize(951, 116));
    // end wxGlade
	
#ifndef __WXMSW__
    m_thumbLabel->SetForegroundColour(wxColour(0, 0, 0));
#endif
	
	// tool bar
	m_toolbar = new wxToolBar(this, -1, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL|wxTB_FLAT|wxTB_DOCKABLE);
	SetToolBar(m_toolbar);
	m_toolbar->AddTool(wxID_NEW,  wxT(""), wxBITMAP(new),  wxNullBitmap,
	  wxITEM_NORMAL, _("New"), wxT(""));
	m_toolbar->AddTool(wxID_OPEN, wxT(""), wxBITMAP(open), wxNullBitmap,
	  wxITEM_NORMAL, _("Open..."), wxT(""));
	m_toolbar->AddTool(wxID_SAVE, wxT(""), wxBITMAP(save), wxNullBitmap,
	  wxITEM_NORMAL, _("Save"), wxT(""));
	m_toolbar->AddTool(MENU_BURN_ID,  wxT(""), wxBITMAP(run),  wxNullBitmap,
	  wxITEM_NORMAL, _("Burn..."), wxT(""));
	m_toolbar->AddSeparator();
	m_toolbar->AddTool(SCALE_MINUS_ID, wxT(""), wxBITMAP(minus), wxNullBitmap,
	  wxITEM_NORMAL, _("Zoom out"), wxT(""));
	const wxString scaleCB_choices[] =
	{
	  _("To page"),
	  _("To width"),
	  _("To height"),
	  _T("10%"),
	  _T("25%"),
	  _T("50%"),
	  _T("100%"),
	  _T("200%"),
	  _T("400%"),
	  _T("800%")
	};
	m_scaleCB = new wxComboBox(m_toolbar, SCALE_BOX_ID, _T(""),
	  wxDefaultPosition, wxSize(72, -1), 10, scaleCB_choices, 0);
	m_toolbar->AddControl(m_scaleCB);
	m_toolbar->AddTool(SCALE_PLUS_ID, _T(""), wxBITMAP(plus), wxNullBitmap,
	  wxITEM_NORMAL, _("Zoom in"), _T(""));
	m_toolbar->Realize();
	
	// dir tree
	m_dirTree->GetTreeCtrl()->GetImageList()->Replace(0, SetDefaultMask(wxBITMAP(folder1)));
    m_dirTree->GetTreeCtrl()->GetImageList()->Replace(1, SetDefaultMask(wxBITMAP(folder2)));
	m_dirTree->SetWindowStyle(m_dirTree->GetWindowStyle() | wxCLIP_CHILDREN);
	
	// thumbnails
    m_thumbnails->SetLabelControl(m_thumbLabel);
	m_thumbnails->EnableDragging();
	
	// menu box
	m_menuBox->SetScale(scFIT_TO_PAGE);
    
    // dir bar
    m_dirBar->SetMargins(4, 4);
    wxVerticalToolbar* vertToolBar = new wxVerticalToolbar(m_dirBar);
    vertToolBar->AddTool(DIR_TREE_BT_ID, _("Directories"), wxBITMAP(files),
	  wxITEM_RADIO);
    vertToolBar->AddTool(BACKGROUNS_BT_ID, _("Backgrounds"), wxNullBitmap,
	  wxITEM_RADIO);
	vertToolBar->AddTool(BUTTONS_BT_ID, _("Buttons"), wxNullBitmap,
	  wxITEM_RADIO);
    vertToolBar->Update();
    m_dirBar->Realize();
}


void MainWin::do_layout()
{
    // begin wxGlade: MainWin::do_layout
    wxBoxSizer* mainWinSizer = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer* thumbSizer = new wxBoxSizer(wxVERTICAL);
    mainWinSizer->Add(m_dirBar, 0, wxEXPAND, 0);
    thumbSizer->Add(m_thumbLabel, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 0);
    thumbSizer->Add(m_thumbnails, 1, wxEXPAND, 0);
    m_thumbPanel->SetAutoLayout(true);
    m_thumbPanel->SetSizer(thumbSizer);
    thumbSizer->Fit(m_thumbPanel);
    thumbSizer->SetSizeHints(m_thumbPanel);
    m_splitterThumb->SplitVertically(m_thumbPanel, m_menuBox, 200);
    m_splitterDir->SplitVertically(m_dirTree, m_splitterThumb, 10);
    m_splitterTitle->SplitHorizontally(m_splitterDir, m_titleThumbnails, 560);
    mainWinSizer->Add(m_splitterTitle, 1, wxEXPAND, 0);
    SetAutoLayout(true);
    SetSizer(mainWinSizer);
    Layout();
    Centre();
    // end wxGlade
	
	// splitters
	m_thumbPanel->SetSizeHints(
	  m_thumbnails->GetMinWidth(), m_thumbnails->GetMinHeight());
	m_splitterDirSash = 200;
	m_splitterDir->Unsplit(m_dirTree);
	m_splitterThumb->SetSashPosition(238);
	m_splitterDir->SetMinimumPaneSize(1);
    m_splitterThumb->SetMinimumPaneSize(1);
	m_splitterTitle->SetMinimumPaneSize(1);
	m_splitterTitleSash = -1;
}

void MainWin::ShowDirTree(bool show)
{
  m_dirBar->ToggleTool(DIR_TREE_BT_ID, show);
  if (show)
  {
    if (!m_splitterDir->IsSplit())
    {
      m_splitterDir->SplitVertically(m_dirTree, m_splitterThumb, m_splitterDirSash);
      m_dirTree->Show(true);
    }
  }
  else
  {
    if (m_splitterDir->IsSplit())
    {
      m_splitterDirSash = m_splitterDir->GetSashPosition();
      
      // animate unsplitting
      m_dirTree->Freeze();
      wxStopWatch sw;
      float step = 0;
      int pos = m_splitterDirSash - 10;
      while (pos > 0)
      {
        m_splitterDir->SetSashPosition(pos);
        while (wxTheApp->Pending())
          wxTheApp->Dispatch();
        if (step == 0)
          step = sw.Time();
        pos -= int(step);
      }
      m_splitterDir->Unsplit(m_dirTree);
      m_dirTree->SetSize(0,0);
      m_dirTree->Thaw();
    }
  }
}

void MainWin::OnDirTreeBt(wxCommandEvent& event)
{
  if (!m_splitterDir->IsSplit())
  {
	ShowDirTree(true);
	wxTreeEvent evt;
	OnDirSelected(evt);
  }
  else
	ShowDirTree(false);
}

void MainWin::OnBackgroundsBt(wxCommandEvent& event)
{
  ShowDirTree(false);
  m_thumbnails->ShowDir(BACKGROUNDS_DIR);
  m_thumbnails->SetCaption(_("Backgrounds"));
}

void MainWin::OnButtonsBt(wxCommandEvent& event)
{
  ShowDirTree(false);
  m_thumbnails->Clear();
  wxString fname = wxFindFirstFile(BUTTONS_DIR + _T("/*.xml"));
  while (!fname.IsEmpty())
  {
	wxThumb* thumb = new wxThumb(fname);
	MenuButton bt(fname);
	thumb->SetImage(bt.GetBitmap(m_thumbnails->GetThumbWidth(),
	  m_thumbnails->GetThumbHeight()).ConvertToImage());
    m_thumbnails->InsertItem(thumb);
    fname = wxFindNextFile();
  }
  m_thumbnails->SortItems();
  m_thumbnails->SetCaption(_("Buttons"));
}

void MainWin::OnDirSelected(wxTreeEvent& event)
{
  if (!m_dirTree)
    return;
  m_thumbnails->ShowDir(m_dirTree->GetPath());
}

void MainWin::OnThumbInfo(wxCommandEvent& event)
{
  wxString text;
  if (m_thumbnails->GetPointed() != -1)
  {
	if (m_dirBar->GetToolState(BACKGROUNS_BT_ID))
	  text = _("Double click to set the background.");
	else if (!m_dirBar->GetToolState(BUTTONS_BT_ID))
	  text = m_thumbnails->GetPointedItem()->GetFilename();
  }
  m_statusbar->SetStatusText(text, 0);
}

void MainWin::OnSetBackground(wxCommandEvent& event)
{
  if (m_thumbnails->GetSelected() != -1 &&
	  m_thumbnails->GetSelectedItem()->GetFilename().AfterLast('.').Lower()!= _T("xml"))
	m_menuBox->SetBackground(m_thumbnails->GetSelectedItem()->GetFilename());
}

void MainWin::OnChangeScale(wxCommandEvent& event)
{
  switch(m_scaleCB->GetSelection())
  {
	case 0: m_menuBox->SetScale(scFIT_TO_PAGE); break;
	case 1: m_menuBox->SetScale(scFIT_TO_WIDTH); break;
	case 2: m_menuBox->SetScale(scFIT_TO_HEIGHT); break;
	case 3: m_menuBox->SetScale(0.1); break;
	case 4: m_menuBox->SetScale(0.25); break;
	case 5: m_menuBox->SetScale(0.5); break;
	case 6: m_menuBox->SetScale(1); break;
	case 7: m_menuBox->SetScale(2); break;
	case 8: m_menuBox->SetScale(4); break;
	case 9: m_menuBox->SetScale(8); break;
	default:;
	  m_scaleCB->SetSelection(0);
	  m_menuBox->SetScale(scFIT_TO_PAGE);
  }
  wxCommandEvent evt(wxEVT_COMMAND_TEXT_ENTER, m_scaleCB->GetId());
  m_toolbar->GetEventHandler()->AddPendingEvent(evt);
}

void MainWin::OnChangeScaleText(wxCommandEvent& event)
{
  double d;
  wxString st = event.GetString();
  if (st.Length() && st[st.Length()-1] == _T('%'))
	st.Remove(st.Length()-1);
  if (st.Length() && st.ToDouble(&d))
  {
	if (d<0) d = -d;
	if (d < 1) d = 1;
	if (d > 800) d = 800;
	m_menuBox->SetScale(d/100);
  }
  else
	d = m_menuBox->GetScaleValue()*100;
  //st = wxString::Format(_T("%0.2f"), d); if (d == (int) d)
  st = wxString::Format(_T("%d"), (int)round(d));
  m_scaleCB->SetValue(st + _T("%"));
}

void MainWin::OnScalePlus(wxCommandEvent& event)
{
  double d = m_menuBox->GetScaleValue()*2;
  if (d > 8) d = 8;
  m_menuBox->SetScale(d);
  wxCommandEvent evt(wxEVT_COMMAND_TEXT_ENTER, m_scaleCB->GetId());
  m_toolbar->GetEventHandler()->AddPendingEvent(evt);
}

void MainWin::OnScaleMinus(wxCommandEvent& event)
{
  double d = m_menuBox->GetScaleValue()/2;
  if (d < 0.01) d = 0.01;
  m_menuBox->SetScale(d);
  wxCommandEvent evt(wxEVT_COMMAND_TEXT_ENTER, m_scaleCB->GetId());
  m_toolbar->GetEventHandler()->AddPendingEvent(evt);
}

void MainWin::OnSplitterTitleSash(wxSplitterEvent &event)
{
  int width, height;
  m_splitterTitle->GetClientSize(&width, &height);
  m_splitterTitleSash = height - m_splitterTitle->GetSashPosition();
}

void MainWin::OnResize(wxSizeEvent &event)
{
  if (m_toolbar)
  {
    wxCommandEvent evt(wxEVT_COMMAND_TEXT_ENTER, SCALE_BOX_ID);
    m_toolbar->GetEventHandler()->AddPendingEvent(evt);
    wxFrame::OnSize(event);
  }
  if (m_splitterTitle && m_splitterTitleSash != -1)
  {
	int width, height;
	m_splitterTitle->GetClientSize(&width, &height);
	m_splitterTitle->SetSashPosition(height - m_splitterTitleSash);
  }
}

void MainWin::OnClose(wxCloseEvent &event)
{
  if (!TestSaved())
	return;
  s_config.SetMainWinLocation(GetRect());
  Destroy();
}

void MainWin::NewDVD(VideoFormat videoFormat)
{
  m_dvd = new DVD;
  Pgc* pgc = new Pgc;
  Menu* menu = new Menu(videoFormat);
  Vob* vob = new Vob(menu);
  pgc->GetVobs().Add(vob);
  m_dvd->GetVmgm().GetPgcs().Add(pgc);
  SetChanged(true);
  UpdateDVD();
  m_anew = true;
}

void MainWin::UpdateDVD()
{
  m_titleThumbnails->SetDVD(m_dvd);
  m_titleThumbnails->UpdateItems();
  Menu* menu = m_dvd->GetVmgm().GetPgcs()[0]->GetVobs()[0]->GetMenu();
  m_menuBox->SetMenu(m_dvd, menu);
  m_menuBox->Update();
}

void MainWin::SetChanged(bool changed)
{
  m_changed = changed;
  m_anew = false;
  m_toolbar->EnableTool(wxID_SAVE, changed);
  m_menubar->GetMenu(0)->Enable(wxID_SAVE, changed);
  wxString fname = m_saveFilename;
  if (!fname.length())
	fname = _("Untitled");
  SetTitle(_T("DVDStyler - ") + fname + (m_changed ? _T("*") : _T("")));
}

void MainWin::OnChanged(wxCommandEvent& event)
{
  SetChanged(true);
}

bool MainWin::TestSaved()
{
  if (m_changed && !m_anew)
  {
	wxString fname = m_saveFilename;
	if (!fname.length())
	  fname = _("Untitled");
	int save = wxMessageBox(
	  wxString::Format(_("The file '%s' is modifed, save changes?"), fname.c_str()),
	  _("Confirm"), wxYES_NO|wxCANCEL|wxICON_QUESTION, this);
	if (save == wxYES)
	{
	  wxCommandEvent event;
	  OnSave(event);
	  if (m_changed)
		return false;
	}
	else if (save == wxCANCEL)
	  return false;
  }
  return true;
}

void MainWin::OnNew(wxCommandEvent& event)
{
  if (!TestSaved())
	return;
  wxString formats[2] = { _T("PAL 720x576"), _T("NTSC 720x480") };
  //Menu::GetVideoFormatStrings()
  wxSingleChoiceDialog dialog(this, _("Create a new dvd"), _("New"),
	2, formats);
  dialog.SetSelection(s_config.GetDefMenuFormat());
  if (dialog.ShowModal() == wxID_OK)
  {
	m_saveFilename = _T("");
	delete m_dvd;
	NewDVD(VideoFormat(dialog.GetSelection()));
  }
}

void MainWin::OnOpen(wxCommandEvent& event)
{
  if (!TestSaved())
	return;
  if (!m_lastDir.length())
	m_lastDir = wxGetHomeDir();
  wxFileDialog dlg( this, _("Open a DVDStyler project file"), m_lastDir,
	_T(""), _T("*.xml"), wxOPEN);
  dlg.Centre();
  if (dlg.ShowModal() == wxID_OK)
  {
    m_saveFilename = dlg.GetPath();
	m_lastDir = wxPathOnly(m_saveFilename);
    if (m_dvd->Open(m_saveFilename))
	{
	  UpdateDVD();
	  SetChanged(false);
	}
	else
      wxLogError( _T("Error opening '%s'"), m_saveFilename.c_str());
  }
}

void MainWin::OnSave(wxCommandEvent& event)
{
  if (m_saveFilename.length())
  {
	if (m_dvd->Save(m_saveFilename))
	  SetChanged(false);
  }
  else
	OnSaveAs(event);
}

void MainWin::OnSaveAs(wxCommandEvent& event)
{
  if (m_dvd->GetTitleSetsCount() ||
      m_dvd->GetMenus()->GetPgcArrayCount())
  {
	if (!m_lastDir.length())
	m_lastDir = wxGetHomeDir();
	wxFileDialog dlg(this, _("Save a DVDStyler project file"), m_lastDir,
	  _T("dvd.xml"), _T("*.xml"), wxSAVE | wxOVERWRITE_PROMPT);
	dlg.Centre();
	if (dlg.ShowModal() == wxID_OK)
	{
	  m_saveFilename = dlg.GetPath();
	  m_lastDir = wxPathOnly(m_saveFilename);
	  if (m_dvd->Save(m_saveFilename))
		SetChanged(false);
	}
  }
  else
	wxLogError(_("Nothing to save"));
}

void MainWin::OnBurn(wxCommandEvent& event)
{
  int vobcnt = 0;
  for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++)
  {
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	for (int pgci = 0; pgci<(int)ts->GetTitles().GetPgcs().Count(); pgci++)
	{
	  if (ts->GetTitles().GetPgcs()[pgci]->GetVobs().Count()>0)
	  {
		vobcnt = 1;
		break;
	  }
	}
  }
  
  if (vobcnt == 0)
  {
	wxMessageBox(
	  _("There are no titles specified. To add a title please drag\n a video file into the bottom section of the window."),
	  _("Burn"), wxOK|wxICON_ERROR, this);
	return;
  }
		
  BurnDlg burnDlg(this);
  if (burnDlg.ShowModal() != wxID_OK)
	return;
  
  // Progress dialog
  ProgressDlg progressDlg(this);
  progressDlg.Start(&burnDlg, m_dvd);
}

void MainWin::OnExit(wxCommandEvent& event)
{
  Close(true);
}

void MainWin::OnSettings(wxCommandEvent& event)
{
  SettingsDlg dlg(this);
  dlg.ShowModal();
}

void MainWin::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  About(this);
}

