/////////////////////////////////////////////////////////////////////////////
// Name:        MainWin.h
// Purpose:     Main window
// Author:      Alex Thuering
// Created:		10.10.2003
// RCS-ID:      $Id: MainWin.h,v 1.8 2004/07/02 08:40:54 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef MAINWIN_H
#define MAINWIN_H

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/dirctrl.h>
#include <wxVillaLib/Thumbnails.h>
#include "MenuBox.h"
#include "TitleThumbnails.h"


// begin wxGlade: dependencies
#include <wx/splitter.h>
// end wxGlade


class MainWin: public wxFrame {
public:
    // begin wxGlade: MainWin::ids
    enum {
        MENU_BURN_ID = 2105,
        MENU_SETTINGS_ID = 2120,
        DIR_TREE_ID = 2000,
        SPLIITTER_TITLE_ID = 2010
    };
    // end wxGlade
	enum {
        DIR_TREE_BT_ID = 2250,
        BACKGROUNS_BT_ID = 2251,
		BUTTONS_BT_ID = 2252,
		SCALE_MINUS_ID= 2261,
		SCALE_PLUS_ID= 2262,
		SCALE_BOX_ID= 2263
    };

    MainWin();

protected:
    // begin wxGlade: MainWin::attributes
    wxMenuBar* m_menubar;
    wxStatusBar* m_statusbar;
    wxToolBar* m_dirBar;
    wxGenericDirCtrl* m_dirTree;
    wxStaticText* m_thumbLabel;
    wxThumbnails* m_thumbnails;
    wxPanel* m_thumbPanel;
    MenuBox* m_menuBox;
    wxSplitterWindow* m_splitterThumb;
    wxSplitterWindow* m_splitterDir;
    TitleThumbnails* m_titleThumbnails;
    wxSplitterWindow* m_splitterTitle;
    // end wxGlade
	
    // begin wxGlade: MainWin::methods
    void set_properties();
    void do_layout();
    // end wxGlade
	
	DVD* m_dvd;
	wxString m_lastDir;
	wxString m_saveFilename;
	bool m_changed;
	bool m_anew;
	
	wxToolBar* m_toolbar;
	wxComboBox* m_scaleCB;
	
	int m_splitterDirSash;
	int m_splitterTitleSash;
	
	void ShowDirTree(bool show);
	void OnDirTreeBt(wxCommandEvent& event);
    void OnBackgroundsBt(wxCommandEvent& event);
	void OnButtonsBt(wxCommandEvent& event);
	
	void OnDirSelected(wxTreeEvent& event);
	void OnThumbInfo(wxCommandEvent& event);
	void OnSetBackground(wxCommandEvent& event);
	
	void NewDVD(VideoFormat videoFormat = vfPAL);
	void UpdateDVD();
	bool GetChanged() { return m_changed; }
	void SetChanged(bool changed);
	void OnChanged(wxCommandEvent& event);
	bool TestSaved();
	
	void OnNew(wxCommandEvent& event);
	void OnOpen(wxCommandEvent& event);
	void OnSave(wxCommandEvent& event);
	void OnSaveAs(wxCommandEvent& event);
	void OnBurn(wxCommandEvent& event);
	void OnExit(wxCommandEvent& event);
	void OnSettings(wxCommandEvent& event);
	void OnAbout(wxCommandEvent& event);
	
	void OnChangeScale(wxCommandEvent& event);
    void OnChangeScaleText(wxCommandEvent& event);
    void OnScalePlus(wxCommandEvent& event);
    void OnScaleMinus(wxCommandEvent& event);
	
	void OnSplitterTitleSash(wxSplitterEvent &event);
	void OnResize(wxSizeEvent &event);
	void OnClose (wxCloseEvent &event);
	
  private:
    DECLARE_EVENT_TABLE()
};


#endif // MAINWIN_H
