/////////////////////////////////////////////////////////////////////////////
// Name:        MenuBox.cpp
// Purpose:     The widget to edit a DVD Menu
// Author:      Alex Thuering
// Created:		11.10.2003
// RCS-ID:      $Id: MenuBox.cpp,v 1.16 2004/07/26 12:40:59 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "MenuBox.h"
#include "MenuPropDlg.h"
#include "ButtonPropDlg.h"
#include "TextPropDlg.h"
#include "MPEG.h"
#include <wxVillaLib/ImageProc.h>
#include <wxVillaLib/utils.h>
#include "math.h"
#include <wx/dnd.h>
#include <wx/utils.h>

//////////////////////////////////////////////////////////////////////////////
////////////////////////////// MenuDnDFile ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

class MenuDnDFile : public wxFileDropTarget
{
  public:
    MenuDnDFile(MenuBox* pOwner) { m_pOwner = pOwner; }

    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
	{
	  x = (int) (x/m_pOwner->GetScaleValue());
	  y = (int) (y/m_pOwner->GetScaleValue());
	  //wxMessageBox(filenames[0]);
	  if (filenames[0].AfterLast('.').Lower() == _T("xml"))
		m_pOwner->AddButton(filenames[0], x, y);
	  else
		m_pOwner->AddImage(filenames[0], x, y);
	  return true;
	}
  protected:
    MenuBox *m_pOwner;
};

//////////////////////////////////////////////////////////////////////////////
/////////////////////////////// MenuBox //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

enum
{
  MENUBOX_PROP_ID = 8440,
  MENUBOX_OBJMENU_DELETE_ID,
  MENUBOX_OBJMENU_PROP_ID,
  MENUBOX_GRIDMENU_ID = 8450,
  MENUBOX_GRIDMENU_ITEM_ID, //51..55
  MENUBOX_ADDMENU_ID = 8460,
  MENUBOX_ADDMENU_TEXT_ID
};

BEGIN_EVENT_TABLE(MenuBox, wxImageBox)
  EVT_LEFT_DOWN(MenuBox::OnMouseLeftButton)
  EVT_LEFT_UP(MenuBox::OnMouseLeftButton)
  EVT_MOTION(MenuBox::OnMouseMove)
  EVT_MENU(MENUBOX_PROP_ID, MenuBox::OnProperties)
  EVT_MENU(MENUBOX_OBJMENU_DELETE_ID, MenuBox::OnObjectDelete)
  EVT_MENU(MENUBOX_OBJMENU_PROP_ID, MenuBox::OnObjectProp)
  EVT_MENU(MENUBOX_GRIDMENU_ITEM_ID, MenuBox::OnGridMenu)
  EVT_MENU(MENUBOX_GRIDMENU_ITEM_ID+1, MenuBox::OnGridMenu)
  EVT_MENU(MENUBOX_GRIDMENU_ITEM_ID+2, MenuBox::OnGridMenu)
  EVT_MENU(MENUBOX_GRIDMENU_ITEM_ID+3, MenuBox::OnGridMenu)
  EVT_MENU(MENUBOX_ADDMENU_TEXT_ID, MenuBox::OnAddText)
END_EVENT_TABLE()

MenuBox::MenuBox(wxWindow *parent, wxWindowID id) : wxImageBox(parent, id)
{
  m_dvd = NULL;
  m_menu = NULL;
  m_selected = -1;
  m_grid = 3;
  SetDropTarget(new MenuDnDFile(this));
  m_objMenu = new wxMenu;
  m_objMenu->Append(MENUBOX_OBJMENU_DELETE_ID, _("&Delete"));
  m_objMenu->Append(MENUBOX_OBJMENU_PROP_ID, _("&Properties..."));
  m_pmenu = new wxMenu;
  m_gridMenu = new wxMenu();
  m_gridMenu->Append(MENUBOX_GRIDMENU_ITEM_ID, _("&none"), wxEmptyString, wxITEM_RADIO);
  m_gridMenu->Append(MENUBOX_GRIDMENU_ITEM_ID+1, _T("8x8"), wxEmptyString, wxITEM_RADIO);
  m_gridMenu->Append(MENUBOX_GRIDMENU_ITEM_ID+2, _T("16x16"), wxEmptyString, wxITEM_RADIO);
  m_gridMenu->Append(MENUBOX_GRIDMENU_ITEM_ID+3, _T("32x32"), wxEmptyString, wxITEM_RADIO);
  m_gridMenu->Check(MENUBOX_GRIDMENU_ITEM_ID+1, true);
  m_pmenu->Append(MENUBOX_GRIDMENU_ID, _("&Grid"), m_gridMenu);
  m_addMenu = new wxMenu();
  m_addMenu->Append(MENUBOX_ADDMENU_TEXT_ID, _("&Text"));
  m_pmenu->Append(MENUBOX_ADDMENU_ID, _("&Add"), m_addMenu);
  m_pmenu->Append(MENUBOX_PROP_ID, _("&Properties..."));
}

void MenuBox::SetMenu(DVD* dvd, Menu* menu)
{
  m_dvd = dvd;
  m_menu = menu;
  if (m_menu->GetBackground().length())
	SetBackground(m_menu->GetBackground());
  else
	SetBackgroundColour(m_menu->GetBackgroundColour());
  m_selected = -1;
  Update();
}

bool MenuBox::SetBackground(wxString fname)
{
  if (!m_menu)
	return false;
  
  wxImage img;
  
  if (wxImage::FindHandler(fname.AfterLast(wxT('.')).Lower(), -1))
  {
	if (!img.LoadFile(fname))
	  return false;
	img.Rescale(m_menu->GetResolution().GetWidth(),
	  m_menu->GetResolution().GetHeight());
  }
  else if (MPEG::IsValid(fname))
  {
	wxBitmap bitmap(m_menu->GetResolution().GetWidth(),
	  m_menu->GetResolution().GetHeight());
	wxMemoryDC dc;
	dc.SelectObject(bitmap);
	dc.SetTextForeground(*wxWHITE);
	dc.SetFont(wxFont(16, wxMODERN, wxNORMAL, wxBOLD, FALSE));
	dc.DrawText(_T("[") + fname + _T("]"), 16, 16);
	dc.SelectObject(wxNullBitmap);
	img = bitmap.ConvertToImage();
  }
  else
  {
	wxLogError(fname + _(": unknown or not supported fromat"));
	return false;
  }
  
  GetImage() = img;
  Update();
  
  m_menu->SetBackground(fname);
  return true;
}

void MenuBox::SetBackgroundColour(wxColour colour)
{
  if (!m_menu)
	return;
  m_image.Create(m_menu->GetResolution().GetWidth(),
	m_menu->GetResolution().GetHeight());
  if (colour.Ok())
	m_image.Replace(0,0,0, colour.Red(), colour.Green(), colour.Blue());
  m_menu->SetBackgroundColour(colour);
}

bool MenuBox::AddImage(wxString fname, int x, int y)
{
  if (m_menu->HasVideoBackground())
  {
	wxLogError(_("You can add only buttons to menu with VIDEO background"));
	return false;
  }
  wxImage img;
  if (!img.LoadFile(fname))
    return false;
  
  m_menu->AddImage(fname, x, y);
  m_selected = m_menu->GetObjectsCount()-1;
  Update();
  return true;
}

bool MenuBox::AddText(wxString text, int x, int y)
{
  if (m_menu->HasVideoBackground())
  {
	wxLogError(_("You can add only buttons to menu with VIDEO background"));
	return false;
  }
  m_menu->AddText(text, x, y);
  m_selected = m_menu->GetObjectsCount()-1;
  Update();
  return true;
}

bool MenuBox::AddButton(wxString fname, int x, int y)
{
  if (!wxFileExists(fname))
    return false;
  
  int n = 1;
  for (int i=0; i<m_menu->GetObjectsCount(); i++)
	if (m_menu->GetObject(i)->GetType() == moBUTTON)
		n++;
  
  m_menu->AddButton(fname, wxString::Format(_T("button%d"), n), x, y);
  m_selected = m_menu->GetObjectsCount()-1;
  Update();
  return true;
}

void MenuBox::Paint()
{
  wxImageBox::Paint();
  
  if (!m_menu)
	return;
  
  wxMemoryDC dc;
  dc.SelectObject(*m_buffer);
  dc.BeginDrawing();
  dc.SetLogicalOrigin((int)(m_bufferX/m_scale), (int)(m_bufferY/m_scale));
  dc.SetUserScale(m_scale, m_scale);
  
  for (int i=0; i<m_menu->GetObjectsCount(); i++)
  {
	MenuObject* obj = m_menu->GetObject(i);
	obj->Draw(dc);
	
	if (i == m_selected)
	{
	  wxRect rect = obj->GetRect();
	  rect.x -= 1;
	  rect.y -= 1;
	  rect.width += 2;
	  rect.height += 2;
	  dc.SetPen(*wxRED_PEN);
	  dc.SetBrush(wxBrush(*wxBLACK, wxTRANSPARENT));
	  dc.DrawRectangle(rect);
	}
  }
  
  dc.EndDrawing();
  dc.SelectObject(wxNullBitmap);
}

void MenuBox::OnMouseLeftButton(wxMouseEvent &event)
{
  int x, y;
  CalcUnscrolledPosition(event.GetX(), event.GetY(), &x, &y);
  m_tt = GetTransformType(x, y);
  m_oldx = x;
  m_oldy = y;
  if (event.LeftDown() && m_tt == ttOut)
  {
	// select Object
	m_selected = -1;
	for (int i=m_menu->GetObjectsCount()-1; i>=0; i--)
	{
	  MenuObject* obj = m_menu->GetObject(i);
	  wxRect rect = ScaleRect(obj->GetRect(), m_scale);
	  if (rect.Inside(x,y))
	  {
		m_selected = i;
		break;
	  }
	}
	event.m_leftDown = false;
	OnMouseMove(event);
	UpdatePaintBox();
  }
}

void MenuBox::OnMouseMove(wxMouseEvent &event)
{
  int x, y;
  CalcUnscrolledPosition(event.GetX(), event.GetY(), &x, &y);
  if (!event.LeftIsDown())
  {
    m_tt = GetTransformType(x, y);
    // change mouse cursor
    switch(m_tt)
    {
      case ttOut: SetCursor(wxCursor(wxCURSOR_ARROW)); break;
#if defined(__WXMSW__) && defined(__WIN95__)
	  case ttIn: SetCursor(wxCursor(_T("wxCURSOR_MOVE"),wxBITMAP_TYPE_CUR_RESOURCE)); break;
	  case ttTL:
      case ttBR: SetCursor(wxCursor(wxCURSOR_SIZENWSE)); break;
      case ttTR:
      case ttBL: SetCursor(wxCursor(wxCURSOR_SIZENESW)); break;
#else
	  case ttIn: SetCursor(wxImage(wxGetAppPath() + _T("/rc/move.cur")));break;
      case ttTL:
      case ttBR: SetCursor(wxImage(wxGetAppPath() + _T("/rc/nwse.cur")));break;
      case ttTR:
      case ttBL: SetCursor(wxImage(wxGetAppPath() + _T("/rc/nesw.cur")));break;
#endif
    }
  }
  else if (m_tt == ttIn)
  {
	// move Object
    int x1 = x - m_oldx;
    int y1 = y - m_oldy;
    MoveObject(x1, y1);
    m_oldx += x1;
    m_oldy += y1;
    UpdatePaintBox();
  }
  else if (m_tt != ttOut) // m_tt=TL,TR,BL,BR
  {
    ResizeObject(x, y, m_tt);
    UpdatePaintBox();
  }
}

void MenuBox::OnMouseRightButton(wxMouseEvent &event)
{
  if (event.RightDown() || !m_menu)
	return;
  // select Object
  int sel_old = m_selected;
  m_selected = -1;
  for (int i=m_menu->GetObjectsCount()-1; i>=0; i--)
  {
	MenuObject* obj = m_menu->GetObject(i);
	wxRect rect = ScaleRect(obj->GetRect(), m_scale);
	if (rect.Inside(event.GetPosition()))
	{
	  m_selected = i;
	  break;
	}
  }
  if (m_selected != sel_old)
	UpdatePaintBox();
  m_menuPos = event.GetPosition();
  if (event.RightUp())
	PopupMenu(m_selected == -1 ? m_pmenu : m_objMenu, event.GetPosition());
}

TransformType MenuBox::GetTransformType(int x, int y)
{
  if (!m_menu || m_selected == -1)
	return ttOut;
  // test all covered object
  for (int i=m_menu->GetObjectsCount()-1; i>m_selected; i--)
  {
	MenuObject* obj = m_menu->GetObject(i);
	wxRect rect = ScaleRect(obj->GetRect(), m_scale);
	if (rect.Inside(x,y))
	  return ttOut; // outside of the selected object
  }
  
  MenuObject* obj = m_menu->GetObject(m_selected);
  wxRect rect = ScaleRect(obj->GetRect(), m_scale);
  
  if (rect.Inside(x,y))
  {
	wxRect rect2 = rect;
	rect2.Deflate(3,3);
    if (!rect2.Inside(x,y))
    {
      int x1 = rect.x + rect.width/2;
      int y1 = rect.y + rect.height/2;
      if ((x < x1) && (y < y1)) 
        return ttTL; // top-left
      else if ((x > x1) && (y < y1)) 
        return ttTR; // top-right
      else if ((x < x1) && (y > y1)) 
        return ttBL; // bottom-left
      else if ((x > x1) && (y > y1)) 
        return ttBR; // bottom-right
    } else
      return ttIn; // inside the photo.cropRect
  }
  
  return ttOut; // outside of the selected object
}

void MenuBox::MoveObject(int& x, int& y)
{
  x = (int) round(x/m_scale);
  y = (int) round(y/m_scale);
  MoveObjectInt(x,y);
  x = (int) round(x*m_scale);
  y = (int) round(y*m_scale);
}

void MenuBox::MoveObjectInt(int& x, int& y)
{
  MenuObject* obj = m_menu->GetObject(m_selected);
  wxRect rect = obj->GetRect();
  rect.x += x; rect.y += y;
  if (rect.GetRight() < 0)
	rect.x = -rect.width+4;
  else if (rect.GetLeft() > m_image.GetWidth())
	rect.x = m_image.GetWidth()-5;
  if (rect.GetBottom() < 0)
	rect.y = -rect.height+4;
  else if (rect.GetTop() > m_image.GetHeight())
	rect.y = m_image.GetHeight()-5;
  
  if (m_grid>0)
  {
	rect.y = rect.y>>m_grid<<m_grid;
	rect.x = rect.x>>m_grid<<m_grid;
	x = rect.x - obj->GetRect().x;
	y = rect.y - obj->GetRect().y;
  }
  
  obj->SetPosition(rect.x, rect.y);
    
  wxCommandEvent evt(EVT_COMMAND_IMAGEBOX_IMAGE_CHANGED, this->GetId());
  GetEventHandler()->ProcessEvent(evt);
}

void MenuBox::ResizeObject(int x, int y, TransformType transformType)
{
  x = (int) round(x/m_scale);
  y = (int) round(y/m_scale);
  ResizeObjectInt(x, y, transformType);
}

void MenuBox::ResizeObjectInt(int x, int y, TransformType transformType)
{
  MenuObject* obj = m_menu->GetObject(m_selected);
  wxRect rect = obj->GetRect();
  
  int sx, sy;
  
  if ((transformType == ttBR) || (transformType == ttTR))
    sx = x - rect.x + 1;
  else
    sx = rect.x + rect.width - x + 1;

  if ((transformType == ttBR) || (transformType == ttBL))
    sy = y - rect.y + 1;
  else
    sy = rect.y + rect.height - y + 1;
  
  if (sx <= 0 && sy <= 0)
	return;
  
  obj->FixSize(sx, sy);

  // set new x, y, width, height
  if ((transformType == ttTL) || (transformType == ttBL))
    rect.x += rect.width - sx;
  if ((transformType == ttTL) || (transformType == ttTR))
    rect.y += rect.height - sy;
  rect.width = sx;
  rect.height = sy;
  
  if (obj->GetRect() != rect)
  {
    obj->SetRect(rect);
    wxCommandEvent evt(EVT_COMMAND_IMAGEBOX_IMAGE_CHANGED, this->GetId());
    GetEventHandler()->ProcessEvent(evt);
  }
}

//////////////////////////////////////////////////////////////////////////////

void MenuBox::OnProperties(wxCommandEvent& event)
{
  MenuPropDlg propDlg(this, m_menu);
  if (propDlg.ShowModal() == wxID_OK)
  {
	if (m_menu->GetBackground().Length())
	  SetBackground(m_menu->GetBackground());
	else
	  SetBackgroundColour(m_menu->GetBackgroundColour());
	Update();
  }
}

void MenuBox::OnGridMenu(wxCommandEvent& event)
{
  m_gridMenu->Check(event.GetId(), true);
  long id = event.GetId() - MENUBOX_GRIDMENU_ITEM_ID;
  m_grid = id;
  if (m_grid>0)
    m_grid += 2;
}

void MenuBox::OnAddText(wxCommandEvent& event)
{
  AddText(_("Text"), (int)(m_menuPos.x/m_scale), (int)(m_menuPos.y/m_scale));
}

void MenuBox::OnObjectDelete(wxCommandEvent& event)
{
  MenuObject* obj = m_menu->GetObject(m_selected); 
  if (obj->GetType() == moBUTTON)
  {
	//update button directions
	int selBtInd = 0;
	for (int i=0; i<m_selected; i++)
	  if (m_menu->GetObject(i)->GetType() == moBUTTON)
		selBtInd++;
	for (int i=0; i<m_menu->GetObjectsCount(); i++)
	{
	  if (m_menu->GetObject(i)->GetType() == moBUTTON)
	  {
		MenuButton* bt = (MenuButton*) m_menu->GetObject(i);
		for (int d=0; d<4; d++)
		  if (bt->GetDirection(d) == selBtInd)
			bt->SetDirection(d, -1);
		  else if (bt->GetDirection(d) > selBtInd)
			bt->SetDirection(d, bt->GetDirection(d)-1);
	  }
	}
  }
  
  m_menu->removeObject(m_selected);
  m_selected = -1;
  UpdatePaintBox();
  
  wxCommandEvent evt(EVT_COMMAND_IMAGEBOX_IMAGE_CHANGED, this->GetId());
  GetEventHandler()->ProcessEvent(evt);
}

void MenuBox::OnObjectProp(wxCommandEvent& event)
{
  MenuObject* obj = m_menu->GetObject(m_selected); 
  if (obj->GetType() == moBUTTON)
  {
	ButtonPropDlg propDlg(this, (MenuButton*) obj, m_menu, m_dvd);
	if (propDlg.ShowModal() == wxID_OK)
	  Update();
  }
  else if (obj->GetType() == moTEXT)
  {
	TextPropDlg propDlg(this, (MenuText*) obj);
	if (propDlg.ShowModal() == wxID_OK)
	  Update();
  }
  else
	wxMessageBox(_T("not impelented yet"));
}

