/////////////////////////////////////////////////////////////////////////////
// Name:        ProgressDlg.cpp
// Purpose:     Progress of generation/burning dialog
// Author:      Alex Thuering
// Created:     14.08.2004
// RCS-ID:      $Id: ProgressDlg.cpp,v 1.2 2004/08/27 09:40:30 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "ProgressDlg.h"
#include "Config.h"
#include "Execute.h"
#include "MPEG.h"
#include <wxVillaLib/utils.h>
#include <wx/dir.h>

#define DATA_DIR(fname) wxGetAppPath() + wxFILE_SEP_PATH + _T("..") + wxFILE_SEP_PATH + fname

void DeleteDir(wxString dir, bool subdirs = false)
{
  if (dir.Last() != wxFILE_SEP_PATH)
	dir += wxFILE_SEP_PATH;
  wxDir d(dir);
  wxString fname;
  while (subdirs && d.GetFirst(&fname, wxEmptyString, wxDIR_DIRS))
	DeleteDir(dir + fname, true);
  while (d.GetFirst(&fname, wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN))
    wxRemoveFile(dir + fname);
  d.Open(wxGetHomeDir());
  wxLogNull log;
  wxRmdir(dir);
}

void DeleteTempDir(wxString dir)
{
  if (dir.Last() != wxFILE_SEP_PATH)
	dir += wxFILE_SEP_PATH;
  wxDir d(dir);
  wxString fname;
  d.GetFirst(&fname, wxEmptyString, wxDIR_DIRS);
  while (1)
  {
	if (fname.Lower() == _T("video_ts") || fname.Lower() == _T("audio_ts"))
	  DeleteDir(dir + fname, false);
	if (!d.GetNext(&fname))
	  break;
  }
  d.GetFirst(&fname, wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN);
  while(1)
  {
	if (fname.Mid(0,4).Lower() == _T("menu") ||
		fname.Mid(0,5).Lower() == _T("title"))
	  wxRemoveFile(dir + fname);
	if (!d.GetNext(&fname))
	  break;
  }
  d.Open(wxGetHomeDir());
  wxLogNull log;
  wxRmdir(dir);
}

BEGIN_EVENT_TABLE(ProgressDlg, wxDialog)
  EVT_BUTTON(wxID_CANCEL, ProgressDlg::OnCancel)
END_EVENT_TABLE()

ProgressDlg::ProgressDlg(wxWindow* parent):
  wxDialog(parent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize,
  wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxTHICK_FRAME)
{
    // begin wxGlade: ProgressDlg::ProgressDlg
    m_summaryLabel = new wxStaticText(this, -1, _("Summary:"));
    m_summaryText = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
    m_summaryGauge = new wxGauge(this, -1, 10, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_PROGRESSBAR|wxGA_SMOOTH);
    m_detailsLabel = new wxStaticText(this, -1, _("Details:"));
    m_detailsText = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
    m_btCancel = new wxButton(this, wxID_CANCEL, _("Cancel"));

    set_properties();
    do_layout();
    // end wxGlade
    m_cancel = false;
}

void ProgressDlg::set_properties()
{
    // begin wxGlade: ProgressDlg::set_properties
    SetTitle(_("Generate DVD"));
    SetSize(wxSize(600, 600));
    // end wxGlade
}

void ProgressDlg::do_layout()
{
    // begin wxGlade: ProgressDlg::do_layout
    wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer* btSizer = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer* panelSizer = new wxBoxSizer(wxVERTICAL);
    panelSizer->Add(m_summaryLabel, 0, wxBOTTOM, 2);
    panelSizer->Add(m_summaryText, 1, wxBOTTOM|wxEXPAND, 8);
    panelSizer->Add(m_summaryGauge, 0, wxBOTTOM|wxEXPAND, 4);
    panelSizer->Add(m_detailsLabel, 0, wxBOTTOM, 2);
    panelSizer->Add(m_detailsText, 1, wxEXPAND, 0);
    mainSizer->Add(panelSizer, 1, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, 10);
    btSizer->Add(20, 20, 1, wxEXPAND, 0);
    btSizer->Add(m_btCancel, 0, 0, 0);
    mainSizer->Add(btSizer, 0, wxALL|wxEXPAND, 10);
    SetAutoLayout(true);
    SetSizer(mainSizer);
    Layout();
    Centre();
    // end wxGlade
}

void ProgressDlg::Update(const wxString& message, bool progress,
  const wxString& details, const wxColour& colour)
{
  if (m_cancel)
	return;
  if (progress)
    m_summaryGauge->SetValue(m_summaryGauge->GetValue() + 1);
  m_summaryText->SetDefaultStyle(wxTextAttr(colour.Ok() ? colour : *wxBLACK));
  m_summaryText->AppendText(message + _T("\n"));
  m_summaryText->ShowPosition(m_summaryText->GetLastPosition());
  UpdateDetails(details.length() ? details : message, false,
	colour.Ok() ? colour : *wxBLACK);
}

void ProgressDlg::UpdateDetails(const wxString& message, bool progress,
  const wxColour& colour)
{
  if (m_cancel)
	return;
  //if (progress)
    //m_detailsGauge->SetValue(m_detailsGauge->GetValue() + 1);
  if (colour.Ok())
	m_detailsText->SetDefaultStyle(wxTextAttr(colour));
  AddDetailText(message + _T("\n"));
  m_detailsText->SetDefaultStyle(wxTextAttr(wxColour(64,64,64)));
}

void ProgressDlg::AddDetailText(const wxString& text)
{
  m_detailsText->AppendText(text);
  m_detailsText->ShowPosition(m_detailsText->GetLastPosition());
}

void ProgressDlg::Failed(const wxString& message)
{
  Update(_("Failed"), false, message, *wxRED);
}

void ProgressDlg::FailedExec(const wxString& command)
{
  Failed(_("Error executing of command: ") + command);
}

void ProgressDlg::OnCancel(wxCommandEvent& WXUNUSED(event))
{
  if (!m_cancel)
  {
	Update(_("Aborted"), false, wxEmptyString, *wxRED);
    End();
  }
  else
    EndModal(wxID_OK);
}

void ProgressDlg::End()
{
  m_btCancel->SetLabel(_("Close"));
  m_cancel = true;
}

bool ProgressDlg::IsCanceled()
{
  wxYield();
  return m_cancel;
}

void ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd)
{
  m_summaryGauge->SetRange(
    4+burnDlg->DoPreview()+burnDlg->DoCreateIso()+burnDlg->DoBurn()+
    (burnDlg->DoFormat() && burnDlg->DoBurn()));
  Show();
  Run(burnDlg, dvd);
  End();
  ShowModal();
}

void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
{
  if (IsCanceled())
	return;
  
  Update(_("Prepare"), false);
  wxString dir = burnDlg->GetTempDir();
  if (dir.Last() != wxFILE_SEP_PATH)
    dir += wxFILE_SEP_PATH;
  
  // clean tmp dir
  UpdateDetails(_("Cleaning temporaly directory"));
  if (wxDir::Exists(dir) && wxFindFirstFile(dir + _T("*"), wxDIR|wxFILE).length())
  {
    if (wxMessageBox(_("The temp directory isn't empty.\nThis direcory must be cleaned.\nContinue?"),
	    _("Burn"), wxYES_NO|wxICON_QUESTION, this) != wxYES)
	{
	  Update(_("Aborted"), false, wxEmptyString, *wxRED);
	  return;
	}
    if (dir.length()<=3)
    {
      Failed(wxString::Format(_("Can't remove directory '%s'"), dir.c_str()));
      return;
    }
    DeleteTempDir(dir);
  }
  // create tmp dir
  if (!wxDir::Exists(dir) && !wxMkdir(dir))
  {
	Failed(wxString::Format(_("Can't create directory '%s'"), dir.c_str()));
    return;
  }
        
  /// Generate menu ///
  if (IsCanceled())
    return;
  wxString menuFile = dir + _T("menu1.mpg");
  dvd->GetVmgm().GetPgcs()[0]->GetVobs()[0]->SetFilename(menuFile);
  Menu* menu = dvd->GetVmgm().GetPgcs()[0]->GetVobs()[0]->GetMenu();
  
  wxString bgFile  = menuFile + _T("_background.jpg");
  wxString bgvFile = menuFile + _T("_background.m2v");
  wxString bgmFile = menuFile + _T("_background.mpg");
  wxString btFile  = menuFile + _T("_buttons.png");
  wxString hlFile  = menuFile + _T("_highlight.png");
  wxString selFile = menuFile + _T("_select.png");
  wxString spuFile = menuFile + _T("_spumux.xml");
  wxString silenceFile = DATA_DIR(_T("silence.mp2"));
  
  bool videoMenu = menu->HasVideoBackground();
  // save background
  Update(_("Generating menu"), true);
  UpdateDetails(_("Prepare"));
  wxImage img;
  if (!videoMenu)
  {
    img = menu->GetImage(mdBACKGROUND).ConvertToImage();
    img.SetOption(_T("quality"), 100);
    img.SaveFile(bgFile);
  }
  // save buttons
  img = menu->GetImage(mdBUTTONS_NORMAL).ConvertToImage();
  img.SaveFile(btFile);
  img = menu->GetImage(mdBUTTONS_HIGHLIGHTED).ConvertToImage();
  img.SaveFile(hlFile);
  img = menu->GetImage(mdBUTTONS_SELECTED).ConvertToImage();
  img.SaveFile(selFile);
  // save spumux
  menu->SaveSpumux(spuFile, btFile, hlFile, selFile);
  
  // convert jpeg to mpeg
  if (videoMenu)
    bgmFile = menu->GetBackground();
  else
  {
    UpdateDetails(_("Converting jpg to mpeg"));
    wxString videoNorm;
    wxString frameRate = s_config.GetFrameRatePAL();
    if (menu->GetVideoFormat() == vfNTSC)
    {
	  videoNorm = _T(" --video-norm n --frame-rate 4");
	  frameRate = s_config.GetFrameRateNTSC();
    }
    wxString cmd = s_config.GetJpeg2MpegCmd();
    cmd.Replace(_T("$FRAME_RATE"), frameRate);
    cmd.Replace(_T("$VIDEO_NORM"), videoNorm);
    cmd.Replace(_T("$FILE_IN"), bgFile);
    cmd.Replace(_T("$FILE_OUT"), bgvFile);
    cmd.Replace(_T("$LOG"), wxEmptyString);
    
    long ret = Exec(cmd);
    if (!ret)
    {
	  FailedExec(cmd);
	  return;
    }
    wxRemoveFile(bgFile);
    
    if (IsCanceled())
      return;
    UpdateDetails(_("Multiplexing audio and video"));
  	cmd = s_config.GetMplexCmd();
    cmd.Replace(_T("$FILE_VIDEO"), bgvFile);
    cmd.Replace(_T("$FILE_AUDIO"), silenceFile);
    cmd.Replace(_T("$FILE_OUT"), bgmFile);
    cmd.Replace(_T("$LOG"), wxEmptyString);
    if (!Exec(cmd))
    {
	  FailedExec(cmd);
	  return;
    }
    wxRemoveFile(bgvFile);
  }
  
  //spumux
  if (IsCanceled())
    return;
  UpdateDetails(_("Multiplexing subtitles (buttons) into mpeg"));
  wxString cmd = s_config.GetSpumuxCmd();
  cmd.Replace(_T("$FILE_CONF"), spuFile);
  if (!Exec(cmd, bgmFile, menuFile))
  {
    FailedExec(cmd);
    return;
  }
  if (!videoMenu)
    wxRemoveFile(bgmFile);
  wxRemoveFile(btFile);
  wxRemoveFile(hlFile);
  wxRemoveFile(selFile);
  wxRemoveFile(spuFile);

  // Test all mpeg-files
  if (IsCanceled())
    return;
  Update(_("Check MPEG-files"), true);
  for (int tsi = 0; tsi<(int)dvd->GetTitlesets().Count(); tsi++)
  {
	Titleset* ts = dvd->GetTitlesets()[tsi];
	for (int pgci = 0; pgci<(int)ts->GetTitles().GetPgcs().Count(); pgci++)
	{
	  Pgc* pgc = ts->GetTitles().GetPgcs()[pgci];
	  for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
	  {
		Vob* vob = pgc->GetVobs()[vobi];
		if (!MPEG::HasNavPacks(vob->GetFilename()))
		{
		  if (IsCanceled())
            return;
		  Update(_("Fix MPEG-file: ") + vob->GetFilename());
		  wxString tmpfname = wxString::Format(_T("title%d-%d-%d"), tsi, pgci, vobi);
		  vob->SetTmpFilename(dir + tmpfname + _T(".vob"));
		  // remove old
		  wxDir d(dir);
		  wxString fname;
		  while (d.GetFirst(&fname, tmpfname + _T("*"), wxDIR_FILES))
			wxRemoveFile(dir + fname);
		  wxString cmd = s_config.GetDemplexCmd();
		  cmd.Replace(_T("$FILE_IN"), vob->GetFilename());
		  cmd.Replace(_T("$FILES_OUT"), dir + tmpfname);
		  cmd.Replace(_T("$LOG"), wxEmptyString);
		  if (!Exec(cmd))
		  {
			FailedExec(cmd);
			return;
		  }
		  cmd = s_config.GetMplexCmd();
		  d.GetFirst(&fname, tmpfname + _T("*"), wxDIR_FILES);
		  cmd.Replace(_T("$FILE_VIDEO"), dir + fname);
		  d.GetNext(&fname);
		  cmd.Replace(_T("$FILE_AUDIO"), dir + fname);
		  cmd.Replace(_T("$FILE_OUT"), vob->GetTmpFilename());
		  cmd.Replace(_T("$LOG"), wxEmptyString);
		  if (!Exec(cmd))
		  {
			FailedExec(cmd);
			return;
		  }
		  while (d.GetFirst(&fname, tmpfname + _T("-*"), wxDIR_FILES))
			wxRemoveFile(dir + fname);
		}
	  }
	}
  }
  
  /// Generate dvd structure ///
  if (IsCanceled())
    return;
  wxString dvdauthFile = dir + _T("dvdauthor.xml");
  dvd->SaveDVDAuthor(dvdauthFile);
  Update(_("Generating DVD"), true);
  cmd = s_config.GetDvdauthorCmd();
  cmd.Replace(_T("$FILE_CONF"), dvdauthFile);
  cmd.Replace(_T("$DIR"), dir);
  cmd.Replace(_T("$LOG"), wxEmptyString);
  if (!Exec(cmd))
  {
    FailedExec(cmd);
    return;
  }
  // remove temp files
  wxRemoveFile(menuFile);
  wxRemoveFile(dvdauthFile);
  wxDir d(dir);
  wxString fname;
  while (d.GetFirst(&fname, _T("*.vob"), wxDIR_FILES))
    wxRemoveFile(dir + fname);

  if (burnDlg->DoPreview())
  {
    if (IsCanceled())
      return;
    Update(_("Start preview"), true);
    wxString cmd = s_config.GetPreviewCmd();
    cmd.Replace(_T("$DIR"), dir);
    cmd.Replace(_T("$LOG"), wxEmptyString);
    if (!Exec(cmd))
      FailedExec(cmd);
    if (burnDlg->DoBurn() && wxMessageBox(_("Burn this video to dvd?"),
         _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
	{
	  Update(_("Aborted"), false, wxEmptyString, *wxRED);
      return;
	}
    if (burnDlg->DoCreateIso() && wxMessageBox(_("Create iso image of this video?"),
         _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
    {
	  Update(_("Aborted"), false, wxEmptyString, *wxRED);
      return;
	}
  }
  
  if (burnDlg->DoCreateIso())
  {
    if (IsCanceled())
      return;
    Update(_("Creating ISO image"), true);
    wxString cmd = s_config.GetIsoCmd();
    cmd.Replace(_T("$DIR"), dir);
    cmd.Replace(_T("$FILE"), burnDlg->GetIsoFile());
    cmd.Replace(_T("$LOG"), wxEmptyString);
    if (!Exec(cmd))
    {
      FailedExec(cmd);
      return;
    }
    // remove tmp dir
  	DeleteTempDir(dir);
  }
  
  if (burnDlg->DoBurn() && burnDlg->DoFormat())
  {
    if (IsCanceled())
      return;
    Update(_("Formating DVD-RW"), true);
    while (1)
    {
      wxString cmd = s_config.GetFormatCmd();
      cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
      cmd.Replace(_T("$LOG"), wxEmptyString);
      if (!Exec(cmd))
      {
	    int repeat = wxMessageBox(_("Formating DVD-RW failed. Try again?"),
	      _("Burn"), wxYES_NO|wxCANCEL|wxICON_QUESTION, this);
        if (repeat == wxYES)
	      continue;
	    else if (repeat == wxNO)
		{
		  Update(_("-> skipped <-"), false, wxEmptyString, wxColour(128, 64, 64));
	      break;
		}
	    else
		{
		  FailedExec(cmd);
	      return;
		}
      }
      break;
    }
  }
  
  if (burnDlg->DoBurn())
  {
    if (IsCanceled())
      return;
    Update(_("Burning"), true);
    wxString cmd = s_config.GetBurnCmd();
    cmd.Replace(_T("$DIR"), dir);
    cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
    cmd.Replace(_T("$LOG"), wxEmptyString);
    if (!Exec(cmd))
    {
      FailedExec(cmd);
      return;
    }
    // remove tmp dir
    DeleteTempDir(dir);
  }
  
  if (IsCanceled())
    return;
  
  if (burnDlg->DoBurn())
    Update(_("Burning was successful."), true, wxEmptyString, wxColour(0,128,0));
  else
    Update(_("Generating was successful."), true, wxEmptyString, wxColour(0,128,0));
}

bool ProgressDlg::Exec(wxString command, wxString inputFile, wxString outputFile)
{
  return Execute(this, command, inputFile, outputFile);
}

