/////////////////////////////////////////////////////////////////////////////
// Name:        SettingsDlg.cpp
// Purpose:     Setting Dlg
// Author:      Alex Thuering
// Created:     27.03.2004
// RCS-ID:      $Id: SettingsDlg.cpp,v 1.6 2004/08/26 11:33:25 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////
 
#include "SettingsDlg.h"
#include "Languages.h"
#include "Menu.h"

SettingsDlg::SettingsDlg(wxWindow* parent):
 PropDlg(parent)
{
  Create(true);
  SetTitle(_("Settings"));
  SetSize(600,-1);
  Center();
}

void SettingsDlg::CreatePropPanel(wxSizer* sizer)
{
  bool def = sizer == NULL;
  wxFlexGridSizer* grid = NULL;
  if (sizer)
    grid = new wxFlexGridSizer(2, 2, 4, 16);
  
  wxString lang = GetLangName(s_config.GetLanguage());
  AddComboProp(grid, _("Language:"), lang, GetLangNames(), wxCB_READONLY);
  wxString menuFormat = Menu::GetVideoFormatStrings()[s_config.GetDefMenuFormat()];
  AddComboProp(grid, _("Default menu format:"), menuFormat, Menu::GetVideoFormatStrings(), wxCB_READONLY);
  AddTextProp(grid, _("Jpeg2Mpeg command:"), s_config.GetJpeg2MpegCmd(def));
  AddTextProp(grid, _("Frame rate PAL:"), s_config.GetFrameRatePAL(def));
  AddTextProp(grid, _("Frame rate NTSC:"), s_config.GetFrameRateNTSC(def));
  AddTextProp(grid, _("Multiplexing command:"), s_config.GetMplexCmd(def));
  AddTextProp(grid, _("Demultiplexing command:"), s_config.GetDemplexCmd(def));
  AddTextProp(grid, _("Spumux command:"), s_config.GetSpumuxCmd(def));
  AddTextProp(grid, _("Dvdauthor command:"), s_config.GetDvdauthorCmd(def));
  AddTextProp(grid, _("Preview command:"), s_config.GetPreviewCmd(def));
  AddTextProp(grid, _("Create ISO command:"), s_config.GetIsoCmd(def));
  AddTextProp(grid, _("Burn command:"), s_config.GetBurnCmd(def));
  AddTextProp(grid, _("Format command:"), s_config.GetFormatCmd(def));
#ifdef __WXMSW__
  AddTextProp(grid, _("CdrecordProDVD key:"), s_config.GetCdrSecurity(def));
#endif
  if (sizer)
  {
    grid->AddGrowableCol(1);
    sizer->Add(grid, 0, wxEXPAND|wxALL, 10);
  }
}

bool SettingsDlg::SetValues()
{
  int i=0;
  if (GetLangCode(GetInt(i++)) != s_config.GetLanguage())
  {
	s_config.SetLanguage(GetLangCode(GetInt(i-1)));
	wxMessageBox(
	  _("Language change will not take effect\nuntil DVDStyler is restarted"),
	  GetTitle(), wxOK|wxICON_INFORMATION, this);
  }
  s_config.SetDefMenuFormat(GetInt(i++));
  s_config.SetJpeg2MpegCmd(GetString(i++));
  s_config.SetFrameRatePAL(GetString(i++));
  s_config.SetFrameRateNTSC(GetString(i++));
  s_config.SetMplexCmd(GetString(i++));
  s_config.SetDemplexCmd(GetString(i++));
  s_config.SetSpumuxCmd(GetString(i++));
  s_config.SetDvdauthorCmd(GetString(i++));
  s_config.SetPreviewCmd(GetString(i++));
  s_config.SetIsoCmd(GetString(i++));
  s_config.SetBurnCmd(GetString(i++));
  s_config.SetFormatCmd(GetString(i++));
#ifdef __WXMSW__
  s_config.SetCdrSecurity(GetString(i++));
#endif
  return true;
}

