/////////////////////////////////////////////////////////////////////////////
// Name:        TitleThumbnails.h
// Purpose:     Title thumbnails
// Author:      Alex Thuering
// Created:		27.01.2003
// RCS-ID:      $Id: TitleThumbnails.cpp,v 1.8 2004/08/30 19:08:57 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "TitleThumbnails.h"
#include "TitlePropDlg.h"
#include "MPEG.h"
#include <wxVillaLib/ThumbnailFactory.h>
#include <wxVillaLib/ImageProc.h>
#include <wx/splitter.h>
#include <wx/dnd.h>
#include <wx/utils.h>

//////////////////////////////////////////////////////////////////////////////
///////////////////////////// TitleDnDFile ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

class TitleDnDFile : public wxFileDropTarget
{
  public:
    TitleDnDFile(TitleThumbnails* pOwner) { m_pOwner = pOwner; }

    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
	{
	  //wxMessageBox(filenames[0]);
	  if (wxImage::FindHandler(filenames[0].AfterLast('.').Lower(), -1))
		return false;
	  if (!MPEG::IsValid(filenames[0]))
	  {
		wxLogError(filenames[0] + _(": not valid mpeg-file"));
		return false;
	  }
	  m_pOwner->InsertTitle(filenames[0]);
	  return true;
	}
	
  protected:
    TitleThumbnails *m_pOwner;
};

//////////////////////////////////////////////////////////////////////////////
/////////////////////////// TitleThumbnails //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

enum
{
  THUMBNAILS_MENU_DEL_ID = 2650,
  THUMBNAILS_MENU_PROP_ID = 2651
};

BEGIN_EVENT_TABLE(TitleThumbnails, wxThumbnails)
  EVT_MENU(THUMBNAILS_MENU_DEL_ID, TitleThumbnails::OnTitleDelete)
  EVT_MENU(THUMBNAILS_MENU_PROP_ID, TitleThumbnails::OnTitleProps)
END_EVENT_TABLE()

DEFINE_EVENT_TYPE(EVT_COMMAND_DVD_CHANGED)

TitleThumbnails::TitleThumbnails(wxWindow* parent, int id):
  wxThumbnails(parent, id)
{
  SetThumbSize(112, 76);
  SetOrientaion(wxTHUMB_HORIZONTAL);
  SetDropTarget(new TitleDnDFile(this));
  m_dvd = NULL;
  
  wxMenu* thmbMenu = new wxMenu;
  thmbMenu->Append(THUMBNAILS_MENU_DEL_ID, _("&Delete"));
  thmbMenu->Append(THUMBNAILS_MENU_PROP_ID, _("&Properties..."));
  SetPopupMenu(thmbMenu);
}

TitleThumbnails::~TitleThumbnails()
{
}

void TitleThumbnails::SetDVD(DVD* dvd)
{
  m_dvd = dvd;
  WX_CLEAR_ARRAY(m_items)
}

void TitleThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index)
{
  if (!m_dvd)
	return;
  wxMemoryDC dc;
  dc.SelectObject(bmp);
  dc.BeginDrawing();  
  int x = m_tBorder/2;
  int y = m_tBorder/2;
  int tw = bmp.GetWidth() - m_tBorder + 2;
  int th = bmp.GetHeight() - m_tBorder + 2;
  // background
  dc.SetPen(wxPen(*wxBLACK,0,wxTRANSPARENT));
  dc.SetBrush(wxBrush(m_backgroundColour, wxSOLID));
  dc.DrawRectangle(0, 0, bmp.GetWidth(), bmp.GetHeight());
  dc.SetBrush(wxBrush(*wxBLACK, wxSOLID));
  dc.DrawRoundedRectangle(x, y, tw, th, 20);
  // image
  wxBitmap img = thumb.GetBitmap(this, m_tWidth, m_tHeight-m_tTextHeight);
  if (index == m_pointed)
  {
    wxImage imgTmp = img.ConvertToImage();
    wxAdjustBrightness(imgTmp, wxRect(0,0,img.GetWidth(),img.GetHeight()),
      POINTED_BRIGHTNESS);
    img = wxBitmap(imgTmp);
  }
  wxRect imgRect(
    x + (m_tWidth-img.GetWidth())/2,
	y + m_tTextHeight + (m_tHeight-m_tTextHeight-img.GetHeight())/2,
    img.GetWidth(), img.GetHeight());      
  dc.DrawBitmap(img, imgRect.x, imgRect.y);
  // outline
  if (index == m_selected)
  {
	dc.SetPen(wxPen(*wxBLUE,0,wxSOLID));
	dc.SetBrush(wxBrush(*wxBLACK, wxTRANSPARENT));
	dc.DrawRectangle(imgRect.x-1, imgRect.y-1, imgRect.width, imgRect.height);
  }
  // draw title caption
  dc.SetTextForeground(*wxWHITE);
  wxString caption = _("Title ") + wxString::Format(_T("%d"), index+1);
  int sw, sh;
  dc.GetTextExtent(caption, &sw, &sh);
  int tx = x + (m_tWidth-sw)/2;
  int ty = y + (m_tTextHeight-sh)/2;
  dc.DrawText(caption, tx, ty);
  // draw caption
  for (int i=0; i<thumb.GetCaptionLinesCount(m_tWidth); i++)
  {
	wxString caption = thumb.GetCaption(m_tWidth, i);
	dc.GetTextExtent(caption, &sw, &sh);
	tx = x + (m_tWidth-sw)/2;
	ty = y + m_tHeight + i*m_tTextHeight + (m_tTextHeight-sh)/2;
	dc.DrawText(caption, tx, ty);
  }
  
  dc.EndDrawing();
  dc.SelectObject(wxNullBitmap);
}

void TitleThumbnails::UpdateShow()
{
  WX_CLEAR_ARRAY(m_items)
  wxThumbnailFactory::ClearQuery(this);
  for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++)
  {
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	for (int pgci = 0; pgci<(int)ts->GetTitles().GetPgcs().Count(); pgci++)
	{
	  Pgc* pgc = ts->GetTitles().GetPgcs()[pgci];
	  for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
	  {
		Vob* vob = pgc->GetVobs()[vobi];
		wxThumb* thumb = new wxThumb(vob->GetFilename(),
		  vob->GetFilename().AfterLast(wxFILE_SEP_PATH).BeforeLast('.'));
		thumb->SetId(((tsi+1)<<24) + ((pgci*2+1)<<8) + vobi);
		m_items.Add(thumb);
	  }
	}
  }
  UpdateProp();
  Refresh();
  ((wxSplitterWindow*)GetParent())->SetSashPosition(
	((wxSplitterWindow*)GetParent())->GetSashPosition()+(m_rows>1?0:1000));
  
  wxCommandEvent evt(EVT_COMMAND_DVD_CHANGED, this->GetId());
  GetEventHandler()->ProcessEvent(evt);
}

void TitleThumbnails::InsertTitle(wxString fname, int pos)
{
  int tsi = 0;
  Vob* vob = new Vob(fname);
  vob->SetChapters(_T("0,15:00,30:00,45:00,1:00:00"));
  Pgc* pgc = new Pgc;
  pgc->SetPostCommands(_T("call vmgm menu 1;"));
  pgc->GetVobs().Add(vob);
  if (!m_dvd->GetTitlesets().Count())
  {
	Titleset* titleset = new Titleset;
	Pgc* pgc = new Pgc;
	pgc->SetPostCommands(_T("jump vmgm menu 1;"));
	titleset->GetMenus().GetPgcs().Add(pgc);
	m_dvd->GetTitlesets().Add(titleset);
  }
  m_dvd->GetTitlesets()[tsi]->GetTitles().GetPgcs().Add(pgc);
  UpdateShow();
}

void TitleThumbnails::OnTitleDelete(wxCommandEvent& event)
{
  wxThumb* thumb = GetItem(GetSelected());
  if (!thumb)
    return;
  int tsi = (thumb->GetId()>>24)-1;
  int pgci = (thumb->GetId()>>8) & 0xFFFF;
  int vobi = thumb->GetId() & 0xFF;
  if (tsi<0)
  {
	//vmgm
  }
  else
  {
	if (tsi>=(int)m_dvd->GetTitlesets().Count())
	  return;
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	if (pgci % 2 == 0)
	{
	  //menu
	  //pgci/=2;
	}
	else
	{
	  //title
	  pgci/=2;
	  if (pgci>=(int)ts->GetTitles().GetPgcs().GetCount())
		return;
	  Pgc* pgc = ts->GetTitles().GetPgcs()[pgci];
	  if (vobi>=(int)pgc->GetVobs().Count())
		return;
	  // remove vob
	  delete pgc->GetVobs()[vobi];
	  pgc->GetVobs().RemoveAt(vobi);
	  // remove title if need
	  if (pgc->GetVobs().Count() == 0)
	  {
		delete ts->GetTitles().GetPgcs()[pgci];
		ts->GetTitles().GetPgcs().RemoveAt(pgci);
	  }
	}
  }
  m_selected = -1;
  UpdateShow();
}

void TitleThumbnails::OnTitleProps(wxCommandEvent& event)
{
  wxThumb* thumb = GetItem(GetSelected());
  if (!thumb)
    return;
  int tsi = (thumb->GetId()>>24)-1;
  int pgci = (thumb->GetId()>>8) & 0xFFFF;
  int vobi = thumb->GetId() & 0xFF;
  if (tsi<0)
  {
	//vmgm
  }
  else
  {
	if (tsi>=(int)m_dvd->GetTitlesets().Count())
	  return;
	Titleset* ts = m_dvd->GetTitlesets()[tsi];
	if (pgci % 2 == 0)
	{
	  //menu
	  //pgci/=2;
	}
	else
	{
	  //title
	  pgci/=2;
	  if (pgci>=(int)ts->GetTitles().GetPgcs().GetCount())
		return;
	  Pgc* pgc = ts->GetTitles().GetPgcs()[pgci];
	  if (vobi>=(int)pgc->GetVobs().Count())
		return;
	  pgc->GetVobs()[vobi];
	  TitlePropDlg propDlg(this, m_dvd, pgc, pgc->GetVobs()[vobi]);
	  propDlg.ShowModal();
	}
  }
}
