/////////////////////////////////////////////////////////////////////////////
// Name:        dvdstyler.cpp
// Purpose:     DVD Authoring Application
// Author:      Alex Thuering
// Created:		10.10.2003
// RCS-ID:      $Id: dvdstyler.cpp,v 1.18 2004/08/27 09:39:53 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "MainWin.h"
#include "About.h"
#include "Config.h"
#include "Languages.h"
#include <wx/config.h>
#include <wxVillaLib/imagjpg.h>
#include <wxVillaLib/utils.h>
#include <wxVillaLib/ThumbnailFactory.h>

#ifndef __WXMSW__ 
#include "rc/dvdstyler.xpm"
#endif

class DVDStyler: public wxApp
{
  public:
    bool OnInit();

  protected:
    wxLocale m_locale;
};

IMPLEMENT_APP(DVDStyler)

bool DVDStyler::OnInit()
{
  //wxLog::SetActiveTarget(new wxLogStderr);
  
  // fix config
  s_config.Init();
  if (s_config.GetVersion() != APP_VERSION)
  {
    wxLogNull log;
    s_config.SetVersion(APP_VERSION);
    s_config.DeleteGroup(_T("Generate"));
    s_config.DeleteGroup(_T("Preview"));
    s_config.DeleteGroup(_T("Iso"));
    s_config.DeleteGroup(_T("Burn"));
    s_config.Flush();
  }
  
  // create a temporary window to set as the top window
  wxFrame *tempWin = new wxFrame(NULL, -1, _T("tempWin"));
  SetTopWindow(tempWin);
  
  // locale
  wxString lang = s_config.GetLanguage();
  s_config.SetLanguage(_T("en")); // save choosed languages only if all ok
  
#ifndef __WXMSW__  // failed on some windows systems
  // Pop up a dialog the first time the program is run
  if (!lang.length())
  {
    m_locale.AddCatalogLookupPathPrefix(wxGetAppPath() + _T("/../locale"));
    m_locale.AddCatalog(_T("dvdstyler"));
    m_locale.Init(wxLANGUAGE_DEFAULT);
	lang = ChooseLanguage(NULL);
  }
#endif

  if (lang != _T("en"))
  {
	m_locale.Init(_T(""), lang, _T(""), true, true);
	m_locale.AddCatalogLookupPathPrefix(wxGetAppPath() + _T("/../locale"));
	m_locale.AddCatalog(_T("dvdstyler"));
  }

  wxInitAllImageHandlers();
  // load new jpeg handler (with scale option)
  if (!wxImage::RemoveHandler(_T("JPEG file")))
    wxMessageBox(_T("Error: Can't remove old JPEG handler"));
  wxImage::InsertHandler(new wxJPGHandler);
  
  wxThumbnailFactory::InitGnome(APP_NAME.mb_str(), APP_VERSION.mb_str(),
	wxTheApp->argc, (char**)wxTheApp->argv);
  
  MainWin* mainWin = new MainWin();
  mainWin->SetIcon(wxICON(dvdstyler));
  SetTopWindow(mainWin); delete tempWin;
  mainWin->Show();
  
  s_config.SetLanguage(lang); // save choosed languages if all ok
  
  return true;
}
