/////////////////////////////////////////////////////////////////////////////
// Name:        PropDlg.h
// Purpose:     Properties dialog (base class for all properties dialogs)
// Author:      Alex Thuering
// Created:     18.11.2003
// RCS-ID:      $Id: PropDlg.h,v 1.5 2004/03/29 10:32:41 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////
 
#ifndef PROP_DLG_H
#define PROP_DLG_H

#include <wx/wx.h>
#include <wx/grid.h>
#include <wx/dynarray.h>

class PropDlg: public wxDialog
{
  public:
    PropDlg(wxWindow *parent);
    virtual ~PropDlg();
	
  protected:
	wxArrayInt m_types;
	wxArrayPtrVoid m_controls;
	wxArrayInt m_groupIds;
	int m_updateIndex; /** index of property by update (reset) */
	int m_currGroupId;
	void* m_currGroupChecker;
	
	void Create(bool resetButton = false);
	/** if sizer != NULL adds properties, else update values */
	virtual void CreatePropPanel(wxSizer* sizer);
	virtual void CreateButtonPane(wxSizer* sizer, bool resetButton = false);

	virtual void Reset();

	// Add properties

	void AddText(wxSizer* sizer, wxString text);

	void AddTextProp(wxSizer* sizer, wxString text, wxString defValue,
	  bool readonly = false);
	void AddStaticTextProp(wxSizer* sizer, wxString text, wxString defValue);
	void AddCheckProp(wxSizer* sizer, wxString text, bool defValue,
	  bool readonly = false, int id=-1);
	void AddComboProp(wxSizer* sizer, const wxString& text,
	  const wxString& defValue, const wxArrayString& choices, long style = 0);
	void SetComboData(wxComboBox* ctrl,
	  const wxString& defValue, const wxArrayString& choices);
	void AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data,
	  const wxString& rowTitle, bool editable);
	void SetGridData(wxGrid* grid, const wxArrayPtrVoid& data,
      const wxString& rowTitle, bool editable);
	void AddFontProp(wxSizer* sizer, wxString text, wxFont font);
	void AddFontProp(wxSizer* sizer, wxString text, wxFontData font);
	void AddColourProp(wxSizer* sizer, wxString text, wxColour colour);

	wxSizer* BeginGroup(wxSizer* sizer, wxString title = wxEmptyString,
	  wxString checkTitle = wxEmptyString, bool defValue = false,
	  bool readonly = false);
	void EndGroup();

	// Get values
	wxString GetString(int index);
	int GetInt(int index);
	bool GetBool(int index);
	wxArrayPtrVoid GetGrid(int index);
	wxFontData GetFont(int index);
	wxColour GetColour(int index);
	
	virtual bool SetValues() { return true; }
	
	void* m_currObject;
	int m_currObjectItem;
  	virtual void OnCancel(wxCommandEvent& event);
    virtual void OnOk(wxCommandEvent& event);
    virtual void OnReset(wxCommandEvent& event);
	virtual void OnSelectFont(wxCommandEvent& event);
	virtual void OnSelectColour(wxCommandEvent& event);
	virtual void OnGroupCheck(wxCommandEvent& event);
	virtual void OnCellLeftClick(wxGridEvent& event);
	virtual void OnCellRightClick(wxGridEvent& event);
	virtual void OnCellChange(wxGridEvent& event);
	virtual void OnRowDelete(wxCommandEvent& event);
	
  private:
    DECLARE_EVENT_TABLE()
};

#endif // PROP_DLG_H
