/////////////////////////////////////////////////////////////////////////////
// Name:        ThumbnailFactory.cpp
// Purpose:     wxThumbnailFactory class
// Author:      Alex Thuering
// Created:		15.02.2003
// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.8 2004/08/29 15:05:11 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#include "ThumbnailFactory.h"
#include "Thumbnails.h"
#include "imagjpg.h"
#include <wx/dir.h>
#include <wx/app.h>

#ifdef GNOME2
#include <glib.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk/gdkdrawable.h>
#include <libgnomeui/gnome-ui-init.h>
#include <libgnomeui/gnome-thumbnail.h>
static GnomeThumbnailFactory* thumbnail_factory = NULL;
#else
#define THUMBNAILS_DIR wxGetHomeDir() + wxFILE_SEP_PATH + _T(".thumbnails2")
#define thumbnailerCmd _T("totem-video-thumbnailer \"$FILE_IN\" \"$FILE_OUT\"")
#ifdef __WXGTK__
#include <sys/types.h>
#include <sys/wait.h>
#endif
#endif

#if defined(__WXMSW__) && defined(__WIN95__)
#undef  wxBITMAP
#define wxBITMAP(icon) wxBitmap(icon##_xpm)
#endif
#include "rc/gnome-fs-loading-icon.xpm"
#include "rc/gnome-mime-video-mpeg.xpm"

class ThumbInfo
{
  public:
	ThumbInfo(wxString filename, wxWindow* parent)
	{
	  this->filename = filename;
	  this->parent = parent;
#ifdef GNOME2
	  pixbuf = NULL;
#endif
	}
	
	~ThumbInfo()
	{
#ifdef GNOME2
	  if (pixbuf)
		g_object_unref(pixbuf);
#endif
	}
	
	wxString filename;
	wxWindow* parent;
	time_t mtime;
#ifdef GNOME2
	char* uri;
	char* mime_type;
	GdkPixbuf* pixbuf;
#else
	wxString uri;
#endif
};

ThumbInfoArray wxThumbnailFactory::m_query;
wxMutex wxThumbnailFactory::m_queryMutex;
wxThumbnailFactory* wxThumbnailFactory::thread = NULL;
#ifdef GNOME2
int wxThumbnailFactory::canGenerate = 1;
#else
int wxThumbnailFactory::canGenerate = -1;
#endif

void wxThumbnailFactory::InitGnome(const char* appName, const char* appVersion,
  int argc, char** argv)
{
#ifdef GNOME2
  gnome_init(appName, appVersion, argc, argv);
  gnome_vfs_init();
  thumbnail_factory = gnome_thumbnail_factory_new(GNOME_THUMBNAIL_SIZE_NORMAL);
#endif
}

wxImage wxThumbnailFactory::GetThumbnail(wxWindow* parent,
 wxString filename, int width, int height)
{
  wxImage img;
  
  if (!filename.length())
	return img;
  
  if (wxImage::FindHandler(filename.AfterLast('.').Lower(), -1))
  {
	wxLogNull log;
	//img.SetOption(_T("max_width"), tw);
	//img.SetOption(_T("max_height"), th);
	((wxJPGHandler*) img.FindHandler(_T("JPEG file")))->m_maxWidth = width;
	((wxJPGHandler*) img.FindHandler(_T("JPEG file")))->m_maxHeight = height;
	img.LoadFile(filename);
	((wxJPGHandler*) img.FindHandler(_T("JPEG file")))->m_maxWidth = -1;
	((wxJPGHandler*) img.FindHandler(_T("JPEG file")))->m_maxHeight = -1;
	return img;
  }
  
  // get thumbnail of video file
  ThumbInfo* info = GetThumbInfo(filename, parent);
  img = LoadThumbnail(*info);
  
  if (!img.Ok() && CanThumbnail(*info)) // generate
  {
#ifndef GNOME2
	if (canGenerate == -1)
	{
	  wxLogNull log;
	  canGenerate = 0;
	  canGenerate = wxExecute(thumbnailerCmd) > 0;
	  if (canGenerate && !wxDir::Exists(THUMBNAILS_DIR))
	  {
		wxMkdir(THUMBNAILS_DIR);
		wxMkdir(THUMBNAILS_DIR + wxFILE_SEP_PATH + _T("normal"));
		wxMkdir(THUMBNAILS_DIR + wxFILE_SEP_PATH + _T("fail"));
	  }
	}
#endif
	if (canGenerate)
	{
	  m_queryMutex.Lock();
	  m_query.Add(info);
	  if (!thread) // start thread
	  {
		thread = new wxThumbnailFactory;
		thread->Create();
		thread->Run();
	  }
	  m_queryMutex.Unlock();
	  img = wxBITMAP(gnome_fs_loading_icon).ConvertToImage();
	}
	else
	  img = wxBITMAP(gnome_mime_video_mpeg).ConvertToImage();
  }
  else
	delete info;
  return img;
}

wxThread::ExitCode wxThumbnailFactory::Entry()
{
  while (1)
  {
	m_queryMutex.Lock();
	if (!m_query.Count())
	  break;
	ThumbInfo* info = m_query.Item(0);
	m_query.RemoveAt(0);
	m_queryMutex.Unlock();
	
	wxImage img;
	wxString thumbPath = GetThumbPath(*info);
	if (thumbPath)
	  img = LoadThumbnail(*info);
	if (!img.Ok())
	{
	  img = GenerateThumbnail(*info);
	  if (img.Ok())
		SaveThumbnail(*info, img);
	  else
		CreateFailedThumbnail(*info);
	}
	wxCommandEvent evt(EVT_COMMAND_THUMBNAILS_THUMB_CHANGED);
	evt.SetString(info->filename);
	evt.SetClientData(new wxImage(img));
	wxPostEvent(info->parent, evt);
	delete info;
  }
  thread = NULL;
  m_queryMutex.Unlock();
  return 0;
}

void wxThumbnailFactory::ClearQuery(wxWindow* parent)
{
  wxMutexLocker locker(m_queryMutex);
  for (int i=0; i<(int)m_query.Count();)
  {
	if (m_query[i]->parent == parent)
	{
	  delete m_query[i];
	  m_query.RemoveAt(i);
	}
	else
	  i++;
  }
}

//////////////////////////////////////////////////////////////////////////////

#ifdef GNOME2
/*wxImage pixbuf2image(GdkPixbuf* pixbuf)
{
  wxBitmap bitmap(gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf));
  GdkBitmap* gbitmap = bitmap.GetPixmap();
  gdk_draw_pixbuf(gbitmap, NULL, pixbuf, 0,0,0,0, -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
  return bitmap.ConvertToImage();
}*/
#else
#include "thumb_md5.cpp"
#endif

ThumbInfo* wxThumbnailFactory::GetThumbInfo(wxString filename, wxWindow* parent)
{
  ThumbInfo* info = new ThumbInfo(filename, parent);
#ifdef GNOME2
  info->uri = gnome_vfs_get_uri_from_local_path(info->filename.mb_str());
  info->mime_type = "video/mpeg";
  // get mtime;
  info->mtime = 0;
  GnomeVFSFileInfo *file_info = gnome_vfs_file_info_new();
  gnome_vfs_get_file_info(info->uri, file_info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
  if (file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME)
	info->mtime = file_info->mtime;
  gnome_vfs_file_info_unref(file_info);
#else
  info->uri = _T("file://");
  info->uri += wxString(escape_string(info->filename.mb_str()), *wxConvCurrent);
#endif
  return info;
}

wxString wxThumbnailFactory::GetThumbPath(ThumbInfo& info, ThumbType type)
{
#ifdef GNOME2
  char* thumbnail_path =
    gnome_thumbnail_factory_lookup(thumbnail_factory, info.uri, info.mtime);
  if (!thumbnail_path || thumbnail_path[0] != '/')
	return wxEmptyString;
  return wxString(thumbnail_path, *wxConvCurrent);
#else
  unsigned char digest[16];
  thumb_md5(info.uri.mb_str(), digest);
  wxString md5 = wxString(thumb_digest_to_ascii(digest), *wxConvCurrent);
  wxString file = md5 + _T(".png");
  wxString dir = THUMBNAILS_DIR;
  dir += wxFILE_SEP_PATH;
  dir += type == THUMBNAIL_FAILED ? _T("fail") : _T("normal");
  return dir + wxFILE_SEP_PATH + file;
#endif
}

wxImage wxThumbnailFactory::LoadThumbnail(ThumbInfo& info, ThumbType type)
{
  wxImage img;
  wxString thumbPath = GetThumbPath(info, type);
#ifdef GNOME2
  GdkPixbuf* pixbuf = NULL;
  if (thumbPath.Length())
	pixbuf = gdk_pixbuf_new_from_file(thumbPath.mb_str(), NULL);
  if (pixbuf)
  {
	if (gnome_thumbnail_is_valid(pixbuf, info.uri, info.mtime))
	{
	  wxLogNull log;
	  img.LoadFile(thumbPath); // img = pixbuf2image(pixbuf); failed on some systems
	}
	g_object_unref(pixbuf);
  }
#else
  wxLogNull log;
  img.LoadFile(thumbPath);
#endif
  return img;
}

bool wxThumbnailFactory::CanThumbnail(ThumbInfo& info)
{
#ifdef GNOME2
  return gnome_thumbnail_factory_can_thumbnail(
    thumbnail_factory, info.uri, info.mime_type, info.mtime);
#else
  wxString thumbPath = GetThumbPath(info, THUMBNAIL_FAILED);
  return !wxFileExists(thumbPath);
#endif
}

wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info)
{
  wxImage img;
#ifdef GNOME2
  info.pixbuf = gnome_thumbnail_factory_generate_thumbnail(
    thumbnail_factory, info.uri, info.mime_type);
  if (info.pixbuf)
	img.Create(1,1);//img = pixbuf2image(info.pixbuf); failed on some systems
#elif defined(__WXGTK__)
  // use totem-video-thumbnailer to create thumbnails
  wxString thumbPath = GetThumbPath(info);
  wxString cmd = thumbnailerCmd;
  cmd.Replace(_T("$FILE_IN"), info.filename);
  cmd.Replace(_T("$FILE_OUT"), thumbPath);
  long pid = wxExecute(cmd);
  waitpid(pid, NULL, 0);
  wxLogNull log;
  img.LoadFile(thumbPath);
#endif
  return img;
}

bool wxThumbnailFactory::SaveThumbnail(ThumbInfo& info, wxImage& image)
{
#ifdef GNOME2
  gnome_thumbnail_factory_save_thumbnail(thumbnail_factory,
    info.pixbuf, info.uri, info.mtime);
  image.LoadFile(GetThumbPath(info)); // see comment in GenerateThumbnail
  return true;
#else
  return image.SaveFile(GetThumbPath(info), wxBITMAP_TYPE_PNG);
#endif
}

void wxThumbnailFactory::CreateFailedThumbnail(ThumbInfo& info)
{
#ifdef GNOME2
  gnome_thumbnail_factory_create_failed_thumbnail(
    thumbnail_factory, info.uri, info.mtime);
#else
  wxImage image(1,1);
  image.SaveFile(GetThumbPath(info, THUMBNAIL_FAILED), wxBITMAP_TYPE_PNG);
  wxRemoveFile(GetThumbPath(info));
#endif
}
