/////////////////////////////////////////////////////////////////////////////
// Name:        ThumbnailFactory.h
// Purpose:     wxThumbnailFactory class
// Author:      Alex Thuering
// Created:		15.02.2003
// RCS-ID:      $Id: ThumbnailFactory.h,v 1.3 2004/06/15 10:58:24 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef WXVILLALIB_THUMBNAILFACTORY_H
#define WXVILLALIB_THUMBNAILFACTORY_H

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/thread.h>
#include <wx/dynarray.h>

enum ThumbType
{
  THUMBNAIL_NORMAL,
  THUMBNAIL_FAILED
};

class ThumbInfo;

WX_DEFINE_ARRAY(ThumbInfo*, ThumbInfoArray);

class wxThumbnailFactory: public wxThread
{
  public:
	static void InitGnome(const char* appName, const char* appVersion,
	  int argc, char** argv);
	static wxImage GetThumbnail(wxWindow* parent,
	  wxString filename, int width, int height);
	static void ClearQuery(wxWindow* parent);
	
  protected:
	static ThumbInfoArray m_query;
	static wxMutex m_queryMutex;
	static wxThumbnailFactory* thread;
	static int canGenerate;
	wxThread::ExitCode Entry();
	static ThumbInfo* GetThumbInfo(wxString filename, wxWindow* parent);
	static wxImage LoadThumbnail(ThumbInfo& info, ThumbType type = THUMBNAIL_NORMAL);
	static bool CanThumbnail(ThumbInfo& info);
	static wxString GetThumbPath(ThumbInfo& info, ThumbType type = THUMBNAIL_NORMAL);
	static wxImage GenerateThumbnail(ThumbInfo& info);
	static bool SaveThumbnail(ThumbInfo& info, wxImage& image);
	static void CreateFailedThumbnail(ThumbInfo& info);
};

#endif //WXVILLALIB_THUMBNAILFACTORY_H
